_.namespace('marketstar');

/**
 * Constructor
 *
 * @class CertificationDetailView
 * @constructor
 * @extends marketstar.DataView
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.CertificationDetailView = function(el, data) {
    $.extend(marketstar.CertificationDetailView.prototype, Constants);
    $.extend(marketstar.CertificationDetailView.prototype, AjaxMixins);
    $.extend(marketstar.CertificationDetailView.prototype, UtilMixins);
    $.extend(marketstar.CertificationDetailView.prototype, PageMixins);
    this.construct(el, data);
};

marketstar.CertificationDetailView.prototype = Object.create(marketstar.DataView.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.CertificationDetailView.prototype.init = function() {
    this.localStorage = new marketstar.LocalStorageObserverProxy();
    this.trainingsLoadAttempts = 0;

    this.model.set(
            'templatePaths'
            ,{
                certificationTrainingView: '_certificationTrainingView.html'
            }
    );

    this.loadIntoModelFromLocalStorage(
            this.userDataKey
            ,$.proxy(this.loadTemplates, this)
            ,$.proxy(this.redirectToIndex, this)
    );
};


/**
 * Render the page
 *
 * @method templatesReady
 */
marketstar.CertificationDetailView.prototype.templatesReady = function() {
    this.renderCertificationTrainings();
};

/**
 * Load the trainings. Yep, this is duplicated from the trainingsMain.js file. That's because SolutionSet's code does not provide an easy way to
 * to run methods in a class statically. So, when I attempted to call this method in trainingsMain.js after instantiating it, it caused all sorts
 * of problems with missing contexts and thus, null variable issues. So, I've duplicated it here and will attempt to refactor it when we refactor
 * SolutionSet's code. CMS
 *
 * @method getTrainingsForType
 * @async
 * @param typeId {Number}
 */
marketstar.CertificationDetailView.prototype.getTrainings = function() {
    this.trainingsLoadAttempts++;
    var urlParams = {
        urlPrefix:  this.marketstarApiUrlPrefix()
        ,urlPath:   'trainings?lang={lang}'
        ,lang:      this.marketstarDefaultLang
    };

    this.marketstarApiAjaxCall({
        url:       this.buildUrl(urlParams)
        ,type:      'GET'
        ,modelName: 'trainings'
        ,success:   $.proxy(this.getTrainingsSuccess, this)
        ,fail:      $.proxy(this.getTrainingsFail, this)
    });
};

/**
 * Success method for getTrainings(). See note in getTrainings(). CMS
 *
 * @method getTrainingsSuccess
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.CertificationDetailView.prototype.getTrainingsSuccess = function(requestProps, xhr) {
    if(xhr) {
        this.localStorage.save('trainings', this.model.getPath('trainings'), this, null, null);
        this.renderCertificationTrainings();
    }
};

/**
 * Fail callback for getTrainings(). See note in getTrainings(). CMS   s
 *
 * @method getTrainingsFail
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.CertificationDetailView.prototype.getTrainingsFail = function(requestProps, xhr) {
    // TODO: Fail
    console.log(xhr);
};

/**
 * Render the product trainings.
 * @method renderCertificationTrainings
 */
marketstar.CertificationDetailView.prototype.renderCertificationTrainings = function() {
    var localStoreTrainings = this.localStorage.read('certifications'); // Retrieving from localstorage because the local product trainings does not contain the usercompleted information. CMS

    // If there are no trainings in localStorage, load them from the web service. CMS
    /*if(localStoreTrainings == null && this.trainingsLoadAttempts <= 3) {
        this.getTrainings();
        return;
    } */

    var trainings = this.model.get("Trainings");
    if( trainings && trainings.length > 0 ) {
        var i
                ,l
                ,training
                ,renderTarget = $('.training-modules')
                ,template
                ,data
                ,child;

        renderTarget.html('');
        for(i = 0; i < trainings.length; i++) {
            training = trainings[i];

            // if there is a local storage object for the training, load it from there. CMS
            /*for(l = 0; l < localStoreTrainings.length; l++) {
                if(localStoreTrainings[l].Id == training.Id) {
                    training = localStoreTrainings[l];
                    break;
                }
            } */

            template = $(_.template( this.model.getPath('templates.certificationTrainingView'), training ));
            $(template).find('a').attr('data-href','trainingDetail.html?trainingId=' + training.Id + '&certification='+window.location.search.split('=')[1]);
            $(template).on('click', $.proxy(this.redirectToTraining, this));

            data = {
                renderTarget: renderTarget
            };
            child = new marketstar.CertificationTrainingView(
                    template
                    , $.extend(data, training)
            );
            this.addChild(child);
        }
    }
};


marketstar.CertificationDetailView.prototype.redirectToTraining = function(e) {
    location.href = decodeURIComponent($(e.currentTarget).find('a').data('href'));
};


/**
 * Post render() callback.  Contract requirement from DataView.
 * Put anything here that is dependent on the existence of elements in the DOM
 *
 * @method postRender
 */
marketstar.CertificationDetailView.prototype.postRender = function() {
};
