_.namespace('marketstar');

/**
 * Constructor
 *
 * @class Chat
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.Chat = function (el, data) {
    $.extend(marketstar.Chat.prototype, Constants);
    $.extend(marketstar.Chat.prototype, UtilMixins);
    $.extend(marketstar.Chat.prototype, AjaxMixins);
    $.extend(marketstar.Chat.prototype, PageMixins);
    this.construct(el, data);
};

marketstar.Chat.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo.js init()
 *
 * @method init
 */
marketstar.Chat.prototype.init = function () {
    this.localStorage = new marketstar.LocalStorageObserverProxy();
    // set the templates we're using
    this.model.set(
        this.templatePathsModelKey
        , {
            navigation: '_navigation.html', userBar: '_userBar.html', socialLinks: '_socialLinks.html', searchResults: '_searchResult.html'
        }
    );
    // load the user into the model, load templates and start page render if successful, else redirect to the index page
    this.loadIntoModelFromLocalStorage(
        this.userDataKey
        , $.proxy(this.loadTemplates, this)
        , $.proxy(this.redirectToIndex, this)
    );
};

/**
 * onTemplatesReady event handler
 *
 * @method onTemplatesReady
 */
marketstar.Chat.prototype.onTemplatesReady = function () {
	if (Config.chatId.length > 0) {
		$("#velaroChatForm").attr("action", "defaultsurvey.aspx?siteid=" + Config.chatId);
		this.getUserInfo();
	} else {

	}
};

/**
 * call into marketstar api and get user info
 *
 * this includes level, points, store, etc.
 */
marketstar.Chat.prototype.getUserInfo = function () {
    var urlParams = {
        urlPrefix: this.marketstarApiUrlPrefix(), urlPath: this.marketstarApiUserInfoPath
    };
    this.marketstarApiAjaxCall({
        url: this.buildUrl(urlParams), type: 'GET'
        //,modelName: this.userInfoKey
        , success: $.proxy(this.getUserInfoSuccess, this), fail: $.proxy(this.getUserInfoFail, this)
    });
};

/**
 * Success callback of getUserInfo()
 */
marketstar.Chat.prototype.getUserInfoSuccess = function (requestProps, xhr) {
    var userInfo = JSON.parse(xhr.responseText);
    //this.$('#your-points').html(userInfo.User.Currency);
	this.$('#IsUserName').val(userInfo.FirstName+' '+userInfo.LastName);
	this.$('#IsEmail').val(userInfo.EmailAddress);
};

/**
 * Fail callback for getUserInfo()
 */
marketstar.Chat.prototype.getUserInfoFail = function (p) {
    //this.$('#your-points').html('Unkown');
};