_.namespace('marketstar');

/**
 * Constructor
 *
 * @class ContentDetailView
 * @constructor
 * @extends _.ViewControlller
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.ContentDetailView = function(el, data) {
    $.extend(marketstar.ContentDetailView.prototype, Constants);
    $.extend(marketstar.ContentDetailView.prototype, AjaxMixins);
    $.extend(marketstar.ContentDetailView.prototype, UtilMixins);
    $.extend(marketstar.ContentDetailView.prototype, PageMixins);
    this.construct(el, data);
};

// subclass _.ViewController
marketstar.ContentDetailView.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.ContentDetailView.prototype.init = function() {
    this.localStorage = new marketstar.LocalStorageObserverProxy();
    this.parseQueryParametersIntoModel();

    if( !this.model.getPath(this.queryParameterModelKey +'.id') ) {
        this.goBack();
    }

    this.GUID = this.model.getPath(this.queryParameterModelKey +'.id');
    this.localStorage.registerObserver('streams', this, this.updateModelFromLocalStorage);

    //this.$('a.back-arrow').on('click', this.goBack);
    this.$('a.back-arrow').on('click', window.parent.PageMixins.hideOverlay);

    this.model.set(
        this.templatePathsModelKey
        ,{
            navigation: '_navigation.html'
            ,userBar: '_userBar.html'
            ,socialLinks: '_socialLinks.html'
            ,contentDetailGroup:   'content/_contentDetailGroup.html'
            ,contentDetailComment:'content/_contentDetailComment.html'
            ,contentDetailFile:   'content/_contentDetailFile.html'
            ,contentDetailDiscussion:   'content/_contentDetailDiscussion.html'
            ,contentDetailReply:    'content/_contentDetailReply.html'
            ,contentDetailUpdate:   'content/_contentDetailUpdate.html'
            ,contentReply:          'content/_contentReply.html'
            ,contentDetailPoll:   'content/_contentDetailPoll.html'
            ,contentDetailIdea:   'content/_contentDetailIdea.html'
            ,contentDetailDocument:   'content/_contentDetailDocument.html'
            ,contentDetailPost:   'content/_contentDetailPost.html'
            /*,contentDetailDirectMessage:   'content/_contentDetailDirectMessage.html'  //doesn't show on activity stream
            ,contentDetailFavorite:   'content/_contentDetailFavorite.html'
            ,contentDetailTask:   'content/_contentDetailTask.html'

              */
        }
    );

    this.loadIntoModelFromLocalStorage(this.userDataKey
        ,$.proxy(this.loadTemplates, this)
        ,$.proxy(this.redirectToIndex, this)
    );

};

/**
 * on templates ready event handler
 *
 * @method onTtemplatesReady
 */
marketstar.ContentDetailView.prototype.onTemplatesReady = function() {
    this.loadIntoModelFromLocalStorage(
        'homePageActivities'
        ,$.proxy(this.loadContent, this)
    );

    $('ul.user-bar').remove();
    $('div.top-bar').css('height', '0px');
};

/**
 * load content
 */
marketstar.ContentDetailView.prototype.loadContent = function() {

    var obj = this.model.getArrayObject('GUID', this.GUID, 'homePageActivities.list');

    var objURL;
    if(obj && obj.target)
        objURL = obj.target.objectType.indexOf('place') >= 0 || obj.target.objectType.indexOf('space') >= 0 || obj.target.objectType.indexOf('group') >= 0 || obj.target.objectType.indexOf('blog') >= 0 || obj.target.objectType.indexOf('person') >= 0 || obj.target.objectType.indexOf('user') >= 0 ? (obj.jive.parent ? obj.jive.parent.id : obj.object.id) : obj.target.id;
    else
        objURL = obj.jive.parent ? obj.jive.parent.id : obj.object.id;

    this.basicAuthAjaxCall({
        url:        objURL
        ,type:      'GET'
        ,basicAuth: this.model.get(this.userDataKey).authHash
        ,modelName: 'stream'
        ,success:   $.proxy(this.searchSuccess, this)
        ,fail:      $.proxy(this.searchFail, this)
    });

};


marketstar.ContentDetailView.prototype.searchSuccess = function(requestProps, xhr){
    var  i
        ,stream = this.model.get(requestProps.modelName)
        ,template
        ,data
        ,child
        ,that = this
        ,$contentDetailHeader = this.$('.stream-detail-header');

    $contentDetailHeader.empty();

    if(stream.replyCount > 0){
        this.basicAuthAjaxCall({
            url:        (stream.resources.comments ? stream.resources.comments.ref : stream.resources.messages.ref)
            ,type:      'GET'
            ,basicAuth: this.model.get(this.userDataKey).authHash
            ,modelName: 'streamComments'
            ,success:   $.proxy(this.commentSearchSuccess, this)
            ,fail:      function(requestProps, xhr){return;}
        });
    }

    // stuff into local storage, if result of an xhr
    if(xhr) {
        //this.localStorage.save('', product, this, this.updateModelFromLocalStorage);
    }
    var authorId = stream.author ? stream.author.id : stream.creator ?  stream.creator.id : stream.person.id;

    stream.isMine = (this.model.get(this.userDataKey).id == authorId ? true : false);
    stream.published = this.getTimeSince( new Date(), new Date(stream.published.replace('+0000','')) );
    if(stream.resources.likes){//"ideas" don't have like option
        stream.liked = ($.inArray('DELETE',stream.resources.likes.allowed) >= 0 ? true : false);
    }

    switch (stream.type)
    {
        case 'file':
            template = _.template(this.model.getPath('templates.contentDetailFile'), stream);
            child = new marketstar.ContentDetailFile(template, $.extend({renderTarget: $contentDetailHeader}, stream));
            break;
        case 'discussion':
            template = _.template(this.model.getPath('templates.contentDetailDiscussion'), stream);
            child = new marketstar.ContentDetailDiscussion(template, $.extend({renderTarget: $contentDetailHeader}, stream));
            break;
        case 'document':
            template = _.template(this.model.getPath('templates.contentDetailDocument'), stream);
            child = new marketstar.ContentDetailDocument(template, $.extend({renderTarget: $contentDetailHeader}, stream));
            break;
        case 'poll':
            template = _.template(this.model.getPath('templates.contentDetailPoll'), stream);
            child = new marketstar.ContentDetailPoll(template, $.extend({renderTarget: $contentDetailHeader}, stream));
            break;
        case 'post':
            template = _.template(this.model.getPath('templates.contentDetailPost'), stream);
            child = new marketstar.ContentDetailPost(template, $.extend({renderTarget: $contentDetailHeader}, stream));
            break;
        case 'dm':
            template = _.template(this.model.getPath('templates.contentDetailDirectMessage'), stream);
            child = new marketstar.ContentDetailDirectMessage(template, $.extend({renderTarget: $contentDetailHeader}, stream));
            break;
        case 'favorite':
            template = _.template(this.model.getPath('templates.contentDetailFavorite'), stream);
            child = new marketstar.ContentDetailFavorite(template, $.extend({renderTarget: $contentDetailHeader}, stream));
            break;
        case 'task':
            template = _.template(this.model.getPath('templates.contentDetailTask'), stream);
            child = new marketstar.ContentDetailTask(template, $.extend({renderTarget: $contentDetailHeader}, stream));
            break;
        case 'update':
            template = _.template(this.model.getPath('templates.contentDetailUpdate'), stream);
            child = new marketstar.ContentDetailUpdate(template, $.extend({renderTarget: $contentDetailHeader}, stream));
            break;
        case 'message':
            template = _.template(this.model.getPath('templates.contentDetailMessage'), stream);
            child = new marketstar.ContentDetailMessage(template, $.extend({renderTarget: $contentDetailHeader}, stream));
            break;
        case 'comment':
            template = _.template(this.model.getPath('templates.contentDetailComment'), stream);
            child = new marketstar.ContentDetailComment(template, $.extend({renderTarget: $contentDetailHeader}, stream));
            break;
        case 'idea':
            template = _.template(this.model.getPath('templates.contentDetailIdea'), stream);
            child = new marketstar.ContentDetailIdea(template, $.extend({renderTarget: $contentDetailHeader}, stream));
            break;
        case 'group':
            template = _.template(this.model.getPath('templates.contentDetailGroup'), stream);
            child = new marketstar.ContentDetailGroup(template, $.extend({renderTarget: $contentDetailHeader}, stream));
            break;
        /*default:
            template = _.template(this.model.getPath('templates.contentDetailComment'), stream);
            child = new marketstar.StreamDetailComment(stream);
         */
    }

    if (child !== null)
        this.addChild(child);
    window.parent.PageMixins.showOverlay();
};


marketstar.ContentDetailView.prototype.commentSearchSuccess = function(requestProps, xhr){
    var streamComments = this.model.get('streamComments').list;
    var $replyContainer = this.$('.stream-comments');

     $replyContainer.empty();
    // stuff in local storage, register an observer
    //this.localStorage.save(modelName, this.model.get(modelName), this, this.updateModelFromLocalStorage);
    for(var i=0; i < streamComments.length; i++){
        comment = streamComments[i];

        // stuff into local storage, if result of an xhr
        if(xhr) {
            //this.localStorage.save('', product, this, this.updateModelFromLocalStorage);
        }

        comment.isMine = (this.model.get(this.userDataKey).id == comment.author.id ? true : false);
        comment.published = this.getTimeSince( new Date(), new Date(comment.published.replace('+0000','')) );
        comment.liked = ($.inArray('DELETE',comment.resources.likes.allowed) >= 0 ? true : false);

        template = _.template(this.model.getPath('templates.contentDetailReply'), comment);
        child = new marketstar.ContentDetailReply(template, $.extend({renderTarget: $replyContainer}, comment));

        this.addChild(child);

    }
}

