_.namespace('marketstar');

/**
 * Constructor
 * 
 * @class CreateStatusUpdate
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.CreateStatusUpdate = function(el, data) {
  $.extend(marketstar.CreateStatusUpdate.prototype, Constants);
  $.extend(marketstar.CreateStatusUpdate.prototype, AjaxMixins);
  $.extend(marketstar.CreateStatusUpdate.prototype, PageMixins);
  $.extend(marketstar.CreateStatusUpdate.prototype, UtilMixins);
  this.construct(el, data);
};

// subclass _.ViewController
marketstar.CreateStatusUpdate.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo.js init()
 *
 * @method init
 */
marketstar.CreateStatusUpdate.prototype.init = function() {
    
    this.localStorage = new marketstar.LocalStorageObserverProxy();
    // set the templates we're using
    this.model.set(
         this.templatePathsModelKey
        ,{
            navigation: '_navigation.html'
            , userBar: '_userBar.html'
            , socialLinks: '_socialLinks.html'
            , statusUpdate: '_statusUpdate.html'
            , searchResults: '_searchResult.html'
        }
    );
    
    // load the user into the model, load templates and start page render if successful, else redirect to the index page
    this.loadIntoModelFromLocalStorage(
         this.userDataKey
        ,$.proxy(this.loadTemplates, this)
        ,$.proxy(this.redirectToIndex, this)
    );
};

/**
 * onTemplatesReady event handler
 *
 * @method onTemplatesReady
 */
marketstar.CreateStatusUpdate.prototype.onTemplatesReady = function() {
    this.populateSelector();
    this.setupSearchTag();
    $('.tag-search').keyup($.proxy(this.searchKeyUpHandler, this));
    $('.tag-search-submit').on('click', $.proxy(this.searchKeyUpHandler, this));
    this.$('.status-post button').on('click', $.proxy(this.submitStatusUpdate, this));
};

/**
 * Event handler for the <select>.onChange()
 *
 * @method populateSelector
 * @event change
 */
marketstar.CreateStatusUpdate.prototype.populateSelector = function() {
    var modelName = this.createCategoriesModelKey
    var props = {
        selector:           '.selector-container .selector'
        ,coll:              this.createDropDownList
        ,idField:           'Id'
        ,nameField:         'Name'
        ,onChangeHandler:   $.proxy(this.changeCreateScreenHandler, this)
        ,initialLoadFun:    $.proxy(this.loadStatusUpdatePage, this)
        //,defaultElement:  {
        //    Id: 0
        //    ,Name: 'Status Update'
        //}
    };

    // stuff in local storage, register an observer
    this.localStorage.save(modelName, this.model.get(modelName), this, this.updateModelFromLocalStorage);

    //TODO:uncomment when adding other create types
    //this.$('.selector-container').show();

    this.updateUl(props)
};

marketstar.CreateStatusUpdate.prototype.loadStatusUpdatePage = function(){
    var template
        ,data
        ,child;

    template = _.template(this.model.getPath('templates.statusUpdate'));

    data = {
        renderTarget: this.$('.status-update-container')
    };

    child = new marketstar.StatusUpdate(
        template
        ,data
    );

    this.addChild(child);
}

/**
 * Event handler for the status-tag.onClick()
 *
 * @method searchTag
 * @event onClick()
 */
marketstar.CreateStatusUpdate.prototype.setupSearchTag = function() {
    $('.status-tag-search').on('click',  $.proxy(this.showTagSearch, this));
};

marketstar.CreateStatusUpdate.prototype.showTagSearch = function(){
    $('.back-arrow').click(function() {
        $('.tag-search-container').hide();

        //TODO:uncomment when adding other create types
        //$('.selector-container').show();
        $('.status-update-container').show();
        $('.search-item-body').html('');
        $('.tag-search').val('');
        $('.nextPage').remove();
        //that.model.unset('searchResults');
        return;
    });

    this.$('.selector-container').hide();
    this.$('.status-update-container').hide();
    this.$('.tag-search-container').show();
};

marketstar.CreateStatusUpdate.prototype.searchKeyUpHandler = function(e){
    if(e.keyCode == 13 || e.target.className == 'tag-search-submit') {
        $('.search-item-body').empty();
        $('.search-item-body').append('<li class="user-searchResult" id="searching">Searching</li>')

        var urlParams = { // used for building urls, see AjaxMixins.buildUrl
            urlPrefix:  this.jiveApiUrlPrefix()
            ,urlPath:   this.jiveApiUserSearchPath
            ,username:  this.$('.tag-search').val()
        };

        this.searchForUsers(this.buildUrl(urlParams));
    }
}
/**
 * searchForUsers action; asynchronous
 *
 * @method searchForUsers
 */
marketstar.CreateStatusUpdate.prototype.searchForUsers = function(url, successFun, failFun) {

    $('.nextPage').remove();

    this.basicAuthAjaxCall({
        url:        url
        ,type:      'GET'
        ,basicAuth: this.model.get(this.userDataKey).authHash
        ,modelName: 'searchResults'
        ,async:     false
        ,success:   (successFun ? successFun : $.proxy(this.searchSuccess, this))
        ,fail:      (failFun ? failFun :$.proxy(this.searchFail, this))
    });
};

marketstar.CreateStatusUpdate.prototype.searchSuccess = function(requestProps, xhr){
    var  i
        ,user
        ,users = this.model.get(requestProps.modelName)
        ,template
        ,data
        ,child
        ,that = this;

    this.$('#searching').remove();
    if(users.list.length == 0){
        this.searchFail();
        return;
    }

    //this.$('.search-item-body').empty();

    for(i=0; i<users.list.length; i++) {
        user = users.list[i];

        // stuff into local storage, if result of an xhr
        if(xhr) {
            //this.localStorage.save('', product, this, this.updateModelFromLocalStorage);
        }

        template = _.template(this.model.getPath('templates.searchResults'), user);

        data = {
            renderTarget: this.$('.search-item-body')
        };

        child = new marketstar.StatusUpdate(
            template
            ,$.extend(data, user)
        );
        this.addChild(child);
        this.updateImageFromAjax('#'+user.id, user.thumbnailUrl);

        $('.select-user-'+user.id).click(function(){
            $('.tag-search-container').hide();

            //TODO:uncomment when adding other create types
            //$('.selector-container').show();
            $('.status-update-container').show();
            $('.search-item-body').html('');
            $('.tag-search').val('');
            $('.nextPage').remove();
            $('#statusText').insertAtCaret(' @'+this.id+' ');
        });
    }

    if(users.links){
        if(users.links.next){
            var that = this;
            $('.tag-search-results').append('<li class="user-searchResult"><a class="nextPage button" >Show More</a></li>');
            $('.nextPage').click(function(){
                    that.searchForUsers(users.links.next);
            });
        }
    }

};

marketstar.CreateStatusUpdate.prototype.searchFail = function(requestProps, xhr){
    $('.search-item-body').remove('.user-searchResult')
    $('.search-item-body').append('<li class="user-searchResult-none" id="searching"><em>Unable to find any users matching your search. Please search again.</em></li>')
};

marketstar.CreateStatusUpdate.prototype.submitStatusUpdate = function() {
    var statusText =  $('#statusText').val().trim();
    $('.status-post button').attr('disabled', 'disabled').addClass('ui-state-disabled');

    if(statusText || statusText.trim() != '')
    {
        var regex = /@[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])|@[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*/g;
        var mentions =  statusText.match(regex);
        var that = this;

        if (mentions != null) {
            for (var i = 0; i < mentions.length; i++)
            {
                var element = mentions[i];
                var user = element.substring(1);
                var jiveMentionHtml = '<a _jive_internal="true" data-jive-statusinputinteral="true" href="/people/{0}" jiveid="3-{1}"> {2}</a> ';
                var urlParams = { // used for building urls, see AjaxMixins.buildUrl
                    urlPrefix:  that.jiveApiUrlPrefix()
                    ,urlPath:   that.jiveApiLoginPath
                    ,username:  user
                };

                if(that.searchForUsers(that.buildUrl(urlParams),
                    function(requestProps, xhr) {
                        var userName = that.model.get(requestProps.modelName).jive.username
                            ,userId = that.model.get(requestProps.modelName).id
                            ,fullName = that.model.get(requestProps.modelName).name.formatted
                        statusText = statusText.replace(element, jiveMentionHtml.format(userName, userId, fullName));
                    },
                    function(requestProps, xhr) {
                        //TODO: handle failure
                        if(xhr.status == 404)
                        {
                            return;
                        }
                    })
                );
            }
        }

        that.model.data.userData.status = statusText;
        that.postStatusUpdate();
    }
    else {
        this.showError('Text is required to update your status');
        $('.status-post button').removeAttr('disabled').removeClass('ui-state-disabled');
    }
};

marketstar.CreateStatusUpdate.prototype.submitStatusSuccess = function(requestProps, xhr){
    window.location.href = 'homepage.html';
};

marketstar.CreateStatusUpdate.prototype.submitStatusFail = function(requestProps, xhr){
    this.showError('We\'re sorry, but there was a problem attempting to post your status. Please try again later.');
    $('.status-post button').removeAttr('disabled').removeClass('ui-state-disabled');
};

marketstar.CreateStatusUpdate.prototype.postStatusUpdate = function() {
    this.basicAuthAjaxCall({
        url:        this.model.data.userData.resources.self.ref
        ,type:      'PUT'
        ,contentType: 'application/json'
        ,basicAuth: this.model.get(this.userDataKey).authHash
        ,payload:   JSON.stringify(this.model.data.userData)
        ,modelName: 'userData'
        ,success:   $.proxy(this.submitStatusSuccess, this)
        ,fail:      $.proxy(this.submitStatusFail, this)
    });
};
