_.namespace('marketstar');

/**
 * Constructor
 *
 * @class GiftPoints
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.GiftPoints = function (el, data) {
    $.extend(marketstar.GiftPoints.prototype, Constants);
    $.extend(marketstar.GiftPoints.prototype, UtilMixins);
    $.extend(marketstar.GiftPoints.prototype, AjaxMixins);
    $.extend(marketstar.GiftPoints.prototype, PageMixins);
    this.construct(el, data);
};

marketstar.GiftPoints.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo.js init()
 *
 * @method init
 */
marketstar.GiftPoints.prototype.init = function () {
    this.localStorage = new marketstar.LocalStorageObserverProxy();
    // set the templates we're using
    this.model.set(
        this.templatePathsModelKey
        , {
            navigation: '_navigation.html', userBar: '_userBar.html', socialLinks: '_socialLinks.html', searchResults: '_searchResult.html'
        }
    );
    // load the user into the model, load templates and start page render if successful, else redirect to the index page
    this.loadIntoModelFromLocalStorage(
        this.userDataKey
        , $.proxy(this.loadTemplates, this)
        , $.proxy(this.redirectToIndex, this)
    );

};



/**
 * onTemplatesReady event handler
 *
 * @method onTemplatesReady
 */
marketstar.GiftPoints.prototype.onTemplatesReady = function () {
    this.$('button.search').on('click', $.proxy(this.findUser, this));
    this.$('.tag-search-submit').click($.proxy(this.searchKeyUpHandler, this));
    this.$('.tag-search').keyup($.proxy(this.searchKeyUpHandler, this));
    this.$('.gift-points-submit').click($.proxy(this.giftPoints, this));
    this.getUserInfo();
};

/**
 * call into marketstar api and get user info
 *
 * this includes level, points, store, etc.
 */
marketstar.GiftPoints.prototype.getUserInfo = function () {
    var urlParams = {
        urlPrefix: this.marketstarApiUrlPrefix(), urlPath: this.marketstarApiUserInfoPath
    };
    this.marketstarApiAjaxCall({
        url: this.buildUrl(urlParams)
        , type: 'GET'
        //,modelName: this.userInfoKey
        , success: $.proxy(this.getUserInfoSuccess, this)
        , fail: $.proxy(this.getUserInfoFail, this)
    });
};

/**
 * Success callback of getUserInfo()
 */
marketstar.GiftPoints.prototype.getUserInfoSuccess = function (requestProps, xhr) {
    var userInfo = JSON.parse(xhr.responseText);
    this.$('#your-points').html(userInfo.Currency);
};

/**
 * Fail callback for getUserInfo()
 */
marketstar.GiftPoints.prototype.getUserInfoFail = function (p) {
    this.$('#your-points').html('Unkown');
};

marketstar.GiftPoints.prototype.findUser = function () {
    $('.back-arrow').click(function () {
        $('.tag-search-container').hide();
        $('.gift-points-container').show();
        return;
    });

    this.$('.gift-points-container').hide();
    this.$('.tag-search-container').show();
};

marketstar.GiftPoints.prototype.searchKeyUpHandler = function (e) {
    if (e.keyCode == 13 || e.target.className == 'tag-search-submit') {
        $('.search-item-body').empty();
        $('.search-item-body').append('<li class="user-searchResult" id="searching">Searching</li>')

        var urlParams = { // used for building urls, see AjaxMixins.buildUrl
            urlPrefix: this.jiveApiUrlPrefix(), urlPath: this.jiveApiUserSearchPath, username: this.$('.tag-search').val()
        };
        this.findUsers(this.buildUrl(urlParams));
    }
};

marketstar.GiftPoints.prototype.findUsers = function (url, successFun, failFun) {

    $('.nextPage').remove();

    this.basicAuthAjaxCall({
        url: url, type: 'GET', basicAuth: this.model.get(this.userDataKey).authHash, modelName: 'searchResults', async: false, success: (successFun ? successFun : $.proxy(this.searchSuccess, this)), fail: (failFun ? failFun : $.proxy(this.searchFail, this))
    });
};

marketstar.GiftPoints.prototype.searchSuccess = function (requestProps, xhr) {
    var i
        , user
        , users = this.model.get(requestProps.modelName)
        , template
        , data
        , child
        , that = this;

    if (users.list.length == 0) {
        this.searchFail();
        return;
    }

    for (i = 0; i < users.list.length; i++) {
        user = users.list[i];

        template = _.template(this.model.getPath('templates.searchResults'), user);

        data = {
            renderTarget: this.$('.search-item-body')
        };

        child = new marketstar.StatusUpdate(
            template
            , $.extend(data, user)
        );
        this.addChild(child);
        this.updateImageFromAjax('#' + user.id, user.thumbnailUrl);

        $('.select-user-' + user.id).click(function () {
            $('.search-item-body').html('');
            $('.tag-search-container').hide();
            $('.gift-points-container').show();
            $('.selected-user').html(this);
        });
    }

    if (users.links) {
        if (users.links.next) {
            var that = this;
            $('.tag-search-results').append('<li class="user-searchResult"><a class="nextPage button" >Show More</a></li>');
            $('.nextPage').click(function () {
                that.findUsers(users.links.next);
            });
        }
    }
};

marketstar.GiftPoints.prototype.searchFail = function (requestProps, xhr) {
    $('.search-item-body').remove('.user-searchResult')
    $('.search-item-body').append('<li class="user-searchResult-none" id="searching"><em>Unable to find any users matching your search. Please search again.</em></li>')
};

marketstar.GiftPoints.prototype.giftPoints = function () {
    this.$('.gift-points-submit').prop("disabled",true);

    var giftPoints = {
        Points: this.$('#points').val(), FromUserId: this.model.getPath(this.userDataKey + '.id'), ToUserId: this.$('#jive-user-id').val(), Reason: this.$('#reason').val()
    };

    if (!giftPoints.ToUserId || giftPoints.ToUserId.length == 0) {
        this.showError('A user must be selected.');
        return;
    }

    var urlParams = {
        urlPrefix: this.marketstarApiUrlPrefix(), urlPath: 'giftPoints'
    };
    this.marketstarApiAjaxCall({
        url: this.buildUrl(urlParams), type: 'POST', body: giftPoints, success: $.proxy(this.giftPointsSuccess, this), fail: $.proxy(this.giftPointsFail, this)
    });
};

marketstar.GiftPoints.prototype.giftPointsSuccess = function (requestProps, xhr) {
    this.showMessage('Success:  You gifted ' + $('#points').val() + ' points.');
    this.$('#points').val('');
    this.$('.selected-user').html('');
    this.$('#reason').val('');
    this.$('.gift-points-submit').prop("disabled",false);

    var response = JSON.parse(xhr.responseText);
    this.$('#your-points').html(response.FromUser.Currency);
};

marketstar.GiftPoints.prototype.giftPointsFail = function (requestProps, xhr) {
    this.$('.gift-points-submit').prop("disabled",false);
    this.showError(xhr.responseText);
};