_.namespace('marketstar');

/**
 * Constructor
 *
 * @class GiftPointsHistory
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.GiftPointsHistory = function(el, data) {
    $.extend(marketstar.GiftPointsHistory.prototype, Constants);
    $.extend(marketstar.GiftPointsHistory.prototype, UtilMixins);
    $.extend(marketstar.GiftPointsHistory.prototype, AjaxMixins);
    $.extend(marketstar.GiftPointsHistory.prototype, PageMixins);
    this.construct(el, data);
};

marketstar.GiftPointsHistory.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo.js init()
 *
 * @method init
 */
marketstar.GiftPointsHistory.prototype.init = function() {
    this.localStorage = new marketstar.LocalStorageObserverProxy();
    // set the templates we're using
    this.model.set(
         this.templatePathsModelKey
        ,{
            navigation: '_navigation.html'
            , userBar: '_userBar.html'
            , socialLinks: '_socialLinks.html'
            , searchResults: '_searchResult.html'
        }
    );    
    // load the user into the model, load templates and start page render if successful, else redirect to the index page
    this.loadIntoModelFromLocalStorage(
         this.userDataKey
        ,$.proxy(this.loadTemplates, this)
        ,$.proxy(this.redirectToIndex, this)
    );
};

/**
 * onTemplatesReady event handler
 *
 * @method onTemplatesReady
 */
marketstar.GiftPointsHistory.prototype.onTemplatesReady = function() {
    this.$('button.search').on('click', $.proxy(this.findUser, this));
    this.$('.tag-search-submit').click($.proxy(this.searchKeyUpHandler, this));
    this.$('.tag-search').keyup($.proxy(this.searchKeyUpHandler, this));
    this.$('.gift-points-submit').click($.proxy(this.giftPoints, this));
    this.getPointsGifted();
};



marketstar.GiftPointsHistory.prototype.getPointsGifted = function() {
    var urlParams = {
        urlPrefix: this.marketstarApiUrlPrefix()
        ,urlPath: this.marketstarApiPointsGiftedPath
    };
    this.marketstarApiAjaxCall({
        url:       this.buildUrl(urlParams)
        ,type:      'GET'
        ,success:   $.proxy(this.getPointsGiftedSuccess, this)
        ,fail:      $.proxy(this.getPointsGiftedFail, this)
    });
};


/**
 * Success callback of getPointsGifted()
 */
marketstar.GiftPointsHistory.prototype.getPointsGiftedSuccess = function(requestProps, xhr) {
    var history = JSON.parse(xhr.responseText);
    var $container = this.$('.gift-history');
    var items = [];

    if (history.length === 0) {
        items.push('<li class="gifted-points">You have not gifted any points.</li>');
    }
    else {
        $.each(history, function (i, h) {
            items.push('<li class="gifted-points">' +
                '<div class="name"><span class="title">Gifted To:</span> <span class="value">' + h.ToUser.FullName + '</span></div>' +
                '<div class="points"><span class="title">Points Gifted:</span> <span class="value">' + h.Points + '</span></div>' +
                '<div class="reason"><span class="title">Gift Reason:</span> <span class="value">' + h.Reason + '</span></div>' +
                /*'<div>Date: '+h.Created+'</div>'+*/
                '<div class="when"><span class="title">You Sent It:</span> <span class="value">' + moment(h.Created).fromNow() + '</span></div>' +
                '</li>');
        });
    }
    $container.append(items);
};

/**
 * Fail callback for getPointsGifted()
 */
marketstar.GiftPointsHistory.prototype.getPointsGiftedFail = function(p) {
    this.showError('error getting history, please try again later');
    //this.$('.gift-history').html('<li class="ui-error"><em>error getting history, please try again later.</em></li>');
};
