_.namespace('marketstar');

/**
 * Constructor
 *
 * @class Homepage
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.Homepage = function (el, data) {
  $.extend(marketstar.Homepage.prototype, Constants);
  $.extend(marketstar.Homepage.prototype, AjaxMixins);
  $.extend(marketstar.Homepage.prototype, PageMixins);
  $.extend(marketstar.Homepage.prototype, UtilMixins);
  this.construct(el, data);
};

// sublass _.ViewController
marketstar.Homepage.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo.js init()
 *
 * @method init
 */
marketstar.Homepage.prototype.init = function () {
  this.localStorage = new marketstar.LocalStorageObserverProxy();
  $('#promotions').hide();
  // set the templates we're using
  this.model.set(
    this.templatePathsModelKey
    , {
      navigation: '_navigation.html'
      , userBar: '_userBar.html'
      , socialLinks: '_socialLinks.html'
      , appVersion: '_appVersion.html'
      , streamEntry: 'content/_streamEntry.html'
    }
  );

  // TODO: Reload user data (including permissions) from the server and update local storage. CMS

  // load the user into the model, load templates and start page render if successful, else redirect to the index page
  this.loadIntoModelFromLocalStorage(
    this.userDataKey
    , $.proxy(this.loadTemplates, this)
    , $.proxy(this.redirectToIndex, this)
  );

  localStorage.removeItem('homePageActivities');
};

/**
 * Render the page
 *
 * @method templatesReady
 */
marketstar.Homepage.prototype.onTemplatesReady = function () {
  this.getUserInfo();
  this.renderUserProfile();
  this.getUserStreams();
  this.getPromotions();
}

/**
 * Display user profile
 *
 * @method renderUserProfile
 */
marketstar.Homepage.prototype.renderUserProfile = function () {
  this.updateImageFromAjax('img.user-avatar', this.model.getPath(this.userDataKey + '.resources.avatar.ref'));
};

/**
 * Load the user's streams from Jive
 *
 * @method getUserStreams
 * @async
 */
marketstar.Homepage.prototype.getUserStreams = function () {
  var urlParams
    , user = this.model.get(this.userDataKey);

  if (user) {
    urlParams = {
      urlPrefix: this.jiveApiUrlPrefix()
      , urlPath: this.jiveApiUserStreamsPath
      , userId: user.id
    };

    this.basicAuthAjaxCall({
      url: this.buildUrl(urlParams)
      , type: 'GET'
      , basicAuth: this.model.get(this.userDataKey).authHash
      , modelName: 'userStreams'
      , success: $.proxy(this.getUserStreamsSuccess, this)
      , fail: $.proxy(this.getUserStreamsFail, this)
    });
  }
};

marketstar.Homepage.prototype.getPromotions = function () {
  var urlParams = {
    urlPrefix: this.marketstarApiUrlPrefix()
    , urlPath: 'user/promotions?mobile=true'
  };

  this.marketstarApiAjaxCall({
    url: this.buildUrl(urlParams)
    , type: 'GET'
    , modelName: 'promotions'
    , success: $.proxy(this.getPromotionsSuccess, this)
    , fail: $.proxy(this.getPromotionsFail, this)
  });

};

marketstar.Homepage.prototype.getPromotionsSuccess = function (requestProps, xhr) {
  this.localStorage.save('promotions', this.model.get(requestProps.modelName), this, this.updateModelFromLocalStorage, null);
  var promotions = this.model.get(requestProps.modelName);

  if (promotions.length > 0) {
    $.each(promotions, function (i, promo) {
      var x = promo.MobileImageUrl;
      if (x != null) {
        $('.slides').append("<li><a class='view-promotion' href=\"" + promo.URL + "\"><img src=\"" + promo.MobileImageUrl + "\" /></a></li>");
      }
    });
  }
  ;
};

marketstar.Homepage.prototype.getPromotionsFail = function () {
  console.log("Error with Promotions");
};
/**
 * Adds the streams as <option>'s to the <select>.
 * Success callback for getUserStreams
 *
 * @method getUserStreamsSuccess
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.Homepage.prototype.getUserStreamsSuccess = function (requestProps, xhr) {
  var props = {
    selector: '.selector-container .selector'
    , coll: this.model.get('userStreams').list
    , idField: 'id'
    , nameFun: function (element) {
      return element.name == "" ? element.source : element.name
    }
    , onChangeHandler: $.proxy(this.getStreamActivitiesHandler, this)
    , initialLoadFun: $.proxy(this.getStreamActivities, this)
    , defaultElement: {
      id: 0
      , name: this.allStreamsText
    }
  };

  this.updateUl(props);
};

/**
 * Failure callback for getUserStreams
 *
 * @method getUserStreamsFail
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.Homepage.prototype.getUserStreamsFail = function (requestProps, xhr) {

};

/**
 * Event handler for the <select>.onChange()
 *
 * @method getStreamActivitiesHandler
 * @event change
 * @param e {Object}
 */
marketstar.Homepage.prototype.getStreamActivitiesHandler = function (e) {
  this.$('ul.feed').html('');
  this.getStreamActivities(e.target.options[e.target.options.selectedIndex].value);
};

/**
 * Load the contents of the Activity Stream
 *
 * @method getStreamActivities
 * @async
 * @param streamId {Number}
 */
marketstar.Homepage.prototype.getStreamActivities = function (streamId, moreUrl) {
  var urlParams
    , user = this.model.get(this.userDataKey);

  if (user) {
    urlParams = {
      urlPrefix: this.jiveApiUrlPrefix()
      , urlPath: this.jiveApiActivitiesPath //streamId == 0 ? this.jiveApiActivitiesPath : this.jiveApiStreamActivitiesPath
      //,streamId:  streamId
    };

    this.basicAuthAjaxCall({
      url: (moreUrl ? moreUrl : this.buildUrl(urlParams))
      , type: 'GET'
      , basicAuth: this.model.get(this.userDataKey).authHash
      , modelName: 'streams' //+ streamId
      , success: $.proxy(this.getStreamActivitiesSuccess, this)
      , fail: $.proxy(this.getStreamActivitiesFail, this)
    });
  }
};

/**
 * Render <li>'s unto the <ul> containing stream activities using sudo template 'templates.streamEntry'
 * Success callback for getStreamActivities.
 *
 * @method getStreamActivitiesSuccess
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.Homepage.prototype.getStreamActivitiesSuccess = function (requestProps, xhr) {
  $('.nextPage').parent().remove();

  var i
    , ts
    , data
    , child
    , template
    , streamEntry
    , stream = this.model.get(requestProps.modelName);

  //remove jive objects when there is no direct user interaction (likes, comment etc)
  stream.list = $.grep(stream.list,
    function (o, i) {
      return o.object ? o.object.objectType === "jive:space" || o.object.objectType === "jive:profileImage" || o.object.objectType === "jive:person" || o.object == null || o.object.objectType == null : o.object == null;
    }, true);

  if (stream.list && stream.list.length === 0 && stream.links && stream.links.next) {
    this.getStreamActivities(0, stream.links.next);
  }

  for (i = 0; i < stream.list.length; i++) {
    stream.list[i].GUID = this.createUUID();
  }

  //this.$('ul.feed').html(''); // clear out the existing li's

  for (i = 0; i < stream.list.length; i++) {
    streamEntry = stream.list[i];

    if (streamEntry.object) {

      streamEntry.verb = streamEntry.verb.replace('jive:', ''); // clean up the verb
      streamEntry.published = this.getTimeSince(new Date(), new Date(streamEntry.published.replace('+0000', ''))); // Jive's +0000 is problematic

      if (!streamEntry.object) { // handles cases when streamEntry.object is missing
        streamEntry.object = {}
        streamEntry.object.summary = "";
      }

      template = _.template(this.model.getPath('templates.streamEntry'), streamEntry);
      data = {
        renderTarget: this.$('ul.feed')
      }
      child = new marketstar.StreamEntry(
        template
        , $.extend(data, streamEntry)
      );
      this.addChild(child);
    }
  }

  if (stream.links) {
    if (stream.links.next) {
      var that = this;
      $('ul.feed').append('<li><a class="button nextPage">Show More</a></li>');
      $('.nextPage').click(function () {
        that.getStreamActivities(0, stream.links.next);
      });
    }
  }

  this.postRender();

  try {
    var ls = JSON.parse(localStorage.getItem('homePageActivities'));
    this.model.set('stream.list', $.merge(stream.list, ls.list));
  }
  catch (e) {
  }
  // stuff in local storage, register an observer
  this.localStorage.save('homePageActivities', this.model.get(requestProps.modelName), this, this.updateModelFromLocalStorage);
};

/**
 * Fail callback for getStreamActivities
 *
 * @method getStreamActivitiesFail
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.Homepage.prototype.getStreamActivitiesFail = function (requestProps, xhr) {
  // TODO: Fail
};

/**
 * call into marketstar api and get user info
 *
 * this includes level, points, store, etc.
 */
marketstar.Homepage.prototype.getUserInfo = function () {
  var urlParams = {
    urlPrefix: this.marketstarApiUrlPrefix()
    , urlPath: this.marketstarApiUserInfoPath
  };
  this.marketstarApiAjaxCall({
    url: this.buildUrl(urlParams)
    , type: 'GET'
    , modelName: this.userInfoKey
    , success: $.proxy(this.getUserInfoSuccess, this)
    , fail: $.proxy(this.getUserInfoFail, this)
  });
};

/**
 * Success callback of getUserInfo()
 */
marketstar.Homepage.prototype.getUserInfoSuccess = function () {
  var userInfo = this.model.get(this.userInfoKey);
  var pointsNeeded = userInfo.NextLevelPoints - userInfo.Points;
  var percentToNext = Math.round((userInfo.Points / userInfo.NextLevelPoints) * 100);
  this.$('#user-profile .name').html(userInfo.FullName);
  this.$('#user-profile .points').text(userInfo.Points);
  this.$('#user-profile .currency').text(userInfo.Currency);
  this.$('#user-profile p.level').html(userInfo.Level);
  this.$('#user-profile p.next-level span.next-level-name').text(userInfo.NextLevel);
  this.$('#user-profile p.next-level span.next-level-points').text(pointsNeeded);
  //this.$('#user-profile span.percentage-filled').attr('style', 'width:'+percentToNext+'%;');
  this.$('#user-profile span.percentage-filled').delay(200).animate({width: percentToNext + '%'}, 500);

  try {
    this.localStorage.save(Constants.userInfoKey, userInfo, null, null, null);
  } catch (e) {
  }
};

/**
 * Fail callback for getUserInfo()
 */
marketstar.Homepage.prototype.getUserInfoFail = function (p) {
  $('.menu').parent().parent().remove();
  $('.create').parent().parent().remove();
  this.showError("It appears you haven't logged into the website after your account was created.  Please take a moment and login to " + this.jiveUrl + " from your computer in order to complete registration process.", 8500);
  var that = this
  setTimeout(function () {
    that.completeLogout(2)
  }, 3500);
};


marketstar.Homepage.prototype.bindScroller = function () {
  var pullDownEl = document.getElementById('pullDown')
    , pullDownOffset = pullDownEl.offsetHeight
    , that = this;

  this.iScroll = new iScroll('iscroller', {
    useTransition: false,
    topOffset: pullDownOffset,
    checkDOMChanges: true,
    onRefresh: function () {
      if (pullDownEl.className.match('loading')) {
        pullDownEl.className = '';
        pullDownEl.querySelector('.pullDownLabel').innerHTML = 'Pull down to refresh...';
      }
    },
    onBeforeScrollStart: function (e) {
      var nodeType = e.explicitOriginalTarget ? e.explicitOriginalTarget.nodeName.toLowerCase() : (e.target ? e.target.nodeName.toLowerCase() : '');

      if (nodeType != 'select' && nodeType != 'option' && nodeType != 'input' && nodeType != 'textarea') {
        e.preventDefault();
      }
    },
    onScrollMove: function () {
      if (this.y > 5 && !pullDownEl.className.match('flip')) {
        pullDownEl.className = 'flip';
        pullDownEl.querySelector('.pullDownLabel').innerHTML = 'Release to refresh...';
        this.minScrollY = 0;
      } else if (this.y < 5 && pullDownEl.className.match('flip')) {
        pullDownEl.className = '';
        pullDownEl.querySelector('.pullDownLabel').innerHTML = 'Pull down to refresh...';
        this.minScrollY = -pullDownOffset;
      }
    },
    onScrollEnd: function () {
      if (pullDownEl.className.match('flip')) {
        pullDownEl.className = 'loading';
        pullDownEl.querySelector('.pullDownLabel').innerHTML = 'Loading...';
        that.onPullToRefresh();
      }
    }
  });


  this.iScroll = new iScroll('navscroller', {
    useTransition: false,
    checkDOMChanges: true,


    onBeforeScrollStart: function (e) {
      //e.stopPropagation();

      this.maxScrollY = (parseInt($(window).height()) - parseInt($('.nav-container').height() + 10));

      var nodeType = e.explicitOriginalTarget ? e.explicitOriginalTarget.nodeName.toLowerCase() : (e.target ? e.target.nodeName.toLowerCase() : '');

      if (nodeType != 'select' && nodeType != 'option' && nodeType != 'input' && nodeType != 'textarea') {
        e.preventDefault();
      }
    }

  });


};

/**
 * On Pull To Refresh Action
 */
marketstar.Homepage.prototype.onPullToRefresh = function () {
  this.$('ul.feed').html('');

  this.getStreamActivities(parseInt($('.selector option:selected').val()), null);
};

marketstar.Homepage.prototype.postRender = function () {
  this.$('a.view-promotion').on('click', $.proxy(this.openAnchorInAppBrowser, this));
  $('#promotions').show();
  $('#promotions').flexslider({
    animation: "slide",
    pauseOnAction: false, // default setting
    after: function (slider) {
      slider.pause();
      slider.play();
    }
  });


  var that = this;
  $('a.stream-content').off('click').on('click', function (e) {
    that.showLoader();
    $('.overlay').find('iframe').attr('src', $(e.currentTarget).data('url'));
  });

}