_.namespace('marketstar');

/**
 * Constructor
 *
 * @class Index
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 * @uses LocalStorageObserverProxy
 */
marketstar.Index = function (el, data) {
  $.extend(marketstar.Index.prototype, Constants);
  $.extend(marketstar.Index.prototype, AjaxMixins);
  $.extend(marketstar.Index.prototype, UtilMixins);
  $.extend(marketstar.Index.prototype, PageMixins);
  if (IndexFixtureMixins) {
    $.extend(marketstar.Index.prototype, IndexFixtureMixins);
  }
  this.construct(el, data);
};

/**
 * Subclass magic
 */
marketstar.Index.prototype = Object.create(sudo.ViewController.prototype);

/**
 * Sudo init.  Called from within the parent construct() method.
 *
 * @method init
 */
marketstar.Index.prototype.init = function () {
  var that = this;
  this.localStorage = new marketstar.LocalStorageObserverProxy();

  $('.forgot-username').attr('href', this.forgotUserNameURL());
  $('.forgot-password').attr('href', this.forgotPasswordURL());
  $('.signup').attr('href', this.createAccountURL());
  $('input.login-button').on('click touchend', $.proxy(this.login, this));
  $('a.default-link, .signup').on('click', $.proxy(this.openAnchorInAppBrowser, this));

  this.model.set(
    'templatePaths'
    , {
      userBar: '_userBar.html'
    }
  );

  // if the model exists in localstorage the user has already logged in
  this.loadIntoModelFromLocalStorage(
    this.userDataKey
    , function () {
      that.checkUserActive();
      that.loadPermissions();
    }
    , $.proxy(this.loadTemplates, this)
  );
};

/**
Polls Jive to ensure user is still active.  Uses existing local storage userDate resourse to get user id.
When fails, clears data and logs out.
 */
marketstar.Index.prototype.checkUserActive = function () {
    var urlParams
            ,user = this.model.get(this.userDataKey);

    if(user) {
        urlParams = {
            urlPrefix: this.jiveApiUrlPrefix()
            ,urlPath:   "people/"+ user.id
        };

        this.basicAuthAjaxCall({
            url:       this.buildUrl(urlParams)
            ,type:      'GET'
            ,basicAuth: this.model.get(this.userDataKey).authHash
            //,modelName: 'userStreams'
            ,success:   $.proxy(this.getUserSuccess, this)
            ,fail:      $.proxy(this.getUserFail, this)
        });
    }
};

marketstar.Index.prototype.getUserSuccess = function (requestProps, xhr) {
};

marketstar.Index.prototype.getUserFail = function () {
    this.completeLogout(3);
};



/**
 * Sudo templates ready callback
 *
 * @method templatesReady
 */
marketstar.Index.prototype.templatesReady = function () {
  $('.login-page').show();
  this.trackPageView();
  this.renderUserBar(false);
  if (location.search.replace('?', '').split('&') == 'loggedout=true') {
    this.showMessage('Logged out successfully!');
  }
  else if(location.search.replace('?', '').split('&') == 'invalidUser=true'){
    this.showError('This account is no longer active.  Please contact your community manager if you have any questions.', 20000);
  }

  var that = this;
  this.$el.keypress(function (e) {
    if (e.which === 13) {
      that.login();
    }
  });
}

/**
 * Login action; asynchronous
 *
 * @method login
 */
marketstar.Index.prototype.login = function () {
  var username = this.$('#username').val()
    , password = this.$('#password').val()
    , urlParams = {
      urlPrefix: this.jiveApiUrlPrefix, urlPath: this.jiveApiLoginPath, username: username
    };

  if (username == null || username.length == 0) {
    this.showError('Username is required.');
    return;
  }
  else if (password == null || password.length == 0) {
    this.showError('Password is required.');
    return;
  }

  var that = this;
  // login requires basic authentication headers to be set
  this.basicAuthAjaxCall({
    url: this.buildUrl(urlParams),
    type: 'GET',
    username: username,
    password: password,
    modelName: 'userData',
    success: $.proxy(this.loginSuccess, this),
    fail: $.proxy(this.loginFail, this)
  });
};

/**
 * Login ajax success callback
 *
 * @method loginSuccess
 * @param requestProps {Object} hash of the request properties that generated the request
 * @param xhr          {Object} the XmlHttpRequest
 */
marketstar.Index.prototype.loginSuccess = function (requestProps, xhr) {
  var userData = this.model.get('userData');
  if (userData) {
    userData.authHash = requestProps.authHash;

    // TOTAL HACK JOB! Remove the empty jive properties to prevent errors upon updating the user. When hire date was empty, Jive would return an error whenever we attempted
    // to update the user's status, etc. So, I'm removing the empty Jive profile fields. Not sure if this affects other modules/functionality in any way. CMS
    if (userData.jive.profile) {
      for (var i = (userData.jive.profile.length - 1); i >= 0; i--) {
        if (userData.jive.profile[i].value == null || userData.jive.profile[i].value == '') {
          userData.jive.profile.splice(i, 1);
        }
      }
    }

    this.marketstarApiAjaxCall({
        url: this.buildUrl({
          urlPrefix: this.marketstarApiUrlPrefix(),
          urlPath: this.marketstarApiUpdateLoginDatePath,
          userId: userData.id
        })
        ,async: false
        ,type: 'POST'
        ,success: function(e) {
            var t = 0;
        }
        ,fail: function(e) {
            var t = 0;
        }
    });

    this.localStorage.save(Constants.userDataKey, userData, null, null, null);
  }

  this.loadPermissions();
};

/**
 * Load/Update user permissions
 */
marketstar.Index.prototype.loadPermissions = function() {
  this.marketstarApiAjaxCall({
    url: this.buildUrl({
      urlPrefix: this.marketstarApiUrlPrefix(),
      urlPath: this.marketstarApiUserPermissions
    }),
    type: 'GET',
    success: $.proxy(this.permissionsSuccess, this),
    fail: $.proxy(this.permissionsFail, this)
  });
};

/**
 * Permission Success
 * @param requestProps
 * @param xhr
 */
marketstar.Index.prototype.permissionsSuccess = function (requestProps, xhr) {
  var permissions = JSON.parse(xhr.response);
  this.localStorage.save(Constants.permissionsKey, permissions, null, null, null);
  this.loadSecurityGroups();
};

/**
 * Permission Fail
 * @param requestProps
 * @param xhr
 */
marketstar.Index.prototype.permissionsFail = function (requestProps, xhr) {
  this.loadSecurityGroups();
};

/**
 * Load Security Groups from Jive
 */
marketstar.Index.prototype.loadSecurityGroups = function() {
  var userData = this.model.get(Constants.userDataKey);

  this.basicAuthAjaxCall({
    url: userData.resources.securityGroups.ref,
    type: 'GET',
    basicAuth: this.model.get(this.userDataKey).authHash,
    success: $.proxy(this.loadSecurityGroupsSuccess, this),
    fail: $.proxy(this.loadSecurityGroupsFail, this)
  });
};

/**
 * Load Security Groups Success
 * @param requestProps
 * @param xhr
 */
marketstar.Index.prototype.loadSecurityGroupsSuccess = function (requestProps, xhr) {
  try {
    var securityGroups = JSON.parse(this.cleanJiveJson(xhr.response)).list;
    this.localStorage.save(Constants.securityGroupsKey, securityGroups, null, null, null);
  }
  catch (e) { }

  window.location.href = Constants.handleOpenUrl.length > 0 ? Constants.handleOpenUrl : 'homepage.html';
};

/**
 * Load Security Groups Fail
 * @param requestProps
 * @param xhr
 */
marketstar.Index.prototype.loadSecurityGroupsFail = function (requestProps, xhr) {
  window.location.href = 'homepage.html';
};

/**
 * Login ajax fail callback
 *
 * @method loginFail
 * @param requestProps {Object} hash of the request properties that generated the request
 * @param xhr          {Object} the XmlHttpRequest
 */
marketstar.Index.prototype.loginFail = function (requestProps, xhr) {
  this.model.set('userData', undefined);
  this.localStorage.clear();

  try {
    this.showError(JSON.parse(xhr.responseText).message);
  } catch (e) {
    this.showError('Invalid Credentials.');
  }
};

/**
 * Track Google Analytics PageView
 *
 * @method trackPageView
 */
marketstar.Index.prototype.trackPageView = function () {
  $.getScript('js/lib/GALocalStorage.js').done(function () {
    if (!ga_storage) return;
    var pageName = location.pathname.substring(location.pathname.lastIndexOf("/") + 1);
    ga_storage._setAccount(Config.googleAnalyticsTrackingId);
    ga_storage._trackPageview('mobile/' + pageName);
  });
};