_.namespace('marketstar');

/**
 * Constructor
 *
 * @class Liaison
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.Liaison = function(el, data) {
    $.extend(marketstar.Liaison.prototype, Constants);
    $.extend(marketstar.Liaison.prototype, UtilMixins);
    $.extend(marketstar.Liaison.prototype, AjaxMixins);
    $.extend(marketstar.Liaison.prototype, PageMixins);
    this.construct(el, data);
};

marketstar.Liaison.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.Liaison.prototype.init = function() {
    this.localStorage = new marketstar.LocalStorageObserverProxy();

    this.model.set(
        'templatePaths'
        ,{
            navigation: '_navigation.html'
            , userBar: '_userBar.html'
            , liaisonView: '_liaisonView.html'
            , appVersion: '_appVersion.html'
			, socialLinks: '_socialLinks.html'
        }
    );

    this.Liaison = this.model.get('Liaison');
    this.loadIntoModelFromLocalStorage(
        this.userDataKey
        , $.proxy(this.loadTemplates, this)
        , $.proxy(this.redirectToIndex, this)
    );
};

/**
 * onTemplatesReady event handler
 *
 * @method onTemplatesReady
 */
marketstar.Liaison.prototype.onTemplatesReady = function() {
    if(!this.Liaison) {
        this.getLiaison();
    }
    else {
        this.getLiaisonSuccess({modelName: 'Liaison'}, null);
    }
}

/**
 * Get the my rep information. CMS
 */
marketstar.Liaison.prototype.getLiaison = function() {
    var urlParams = {
        urlPrefix:  this.marketstarApiUrlPrefix()
        ,urlPath:    'user/rep'
    };

    this.marketstarApiAjaxCall({
        url:            this.buildUrl(urlParams)
        , type:         'GET'
        , modelName:    'Liaison'
        , success:      $.proxy(this.getLiaisonSuccess, this)
        , fail:         $.proxy(this.getLiaisonFail, this)
    });
}

marketstar.Liaison.prototype.getLiaisonSuccess = function(requestProps, xhr) {
    var liaison = this.model.get(requestProps.modelName)
        ,$template
        ,data
        ,child;

    if(!liaison || !liaison.FirstName || liaison.FirstName == '' || !liaison.LastName || liaison.LastName == '') {
        this.$el.find('.no-rep-info').show();
        this.$el.find('.my-rep-container').hide();
        return;
    }

    $template = $(_.template(this.model.getPath('templates.liaisonView'), liaison))

    data = {
        renderTarget: this.$('.my-rep-container')
    };
    child = new marketstar.LiaisonView(
        $template
        ,$.extend(data, liaison)
    );

    this.addChild(child);

	if (Config.liasonTitle.length > 0) {
		$('h5.liasonTitle').prepend(Config.liasonTitle);
	}

    if(xhr) {
        this.localStorage.save(requestProps.modelName, this.model.get(requestProps.modelName), this, this.updateModelFromLocalStorage, null);
    }
}


marketstar.Liaison.prototype.getLiaisonFail = function(requestProps, xhr) {
    this.$el.find('.no-rep-info').show();
    this.$el.find('.my-rep-container').hide();
    return;

}