_.namespace('marketstar');

/**
 * Constructor
 *
 * @class MissionsMain
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.MissionsMain = function(el, data) {
	$.extend(marketstar.MissionsMain.prototype, Constants);
	$.extend(marketstar.MissionsMain.prototype, UtilMixins);
	$.extend(marketstar.MissionsMain.prototype, AjaxMixins);
	$.extend(marketstar.MissionsMain.prototype, PageMixins);
	this.construct(el, data);
	this.$('.selector').change($.proxy(this.selectorChange, this));
};

marketstar.MissionsMain.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.MissionsMain.prototype.init = function() {
	this.localStorage = new marketstar.LocalStorageObserverProxy();

	this.model.set(
		'templatePaths'
		,{
            navigation: '_navigation.html'
            , userBar: '_userBar.html'
			, socialLinks: '_socialLinks.html'
            , appVersion: '_appVersion.html'
			, mission:   '_mission.html'
            , training: '_achievementList.html'
            , category: 'achievementfilters/_category.html'
            , tag: 'achievementfilters/_tag.html'
            , point: 'achievementfilters/_point.html'
		}
	);

	this.loadIntoModelFromLocalStorage(
		this.userDataKey
		,$.proxy(this.loadTemplates, this)
		,$.proxy(this.redirectToIndex, this)
	);

    this.$('.template-selector').on('click', $.proxy(this.changeLayout, this));
    this.$('.achieveSearch').on('keyup input search', $.proxy(this.filterMissions, this));
    this.$('.achieveSearch').on('blur', $.proxy(this.adjustSearchBar, this));
    this.$('.filtering').on('click', $.proxy(this.filterMissions, this));
    this.$('.achieveFilter').on('click', $.proxy(this.missionFilterHandler, this));
    this.$('.filter-done').on('click', $.proxy(this.missionFilterHandler, this));

};

/**
 * Templates Ready
 *
 * @method onTemplatesReady
 */
marketstar.MissionsMain.prototype.onTemplatesReady = function() {
    this.getChallenges();
    this.adjustSearchBar();
};

/**
 * Used becuase android 4.3 and lower doesn't support certain css stuff
 */
marketstar.MissionsMain.prototype.adjustSearchBar = function() {
    var searchbarWidth = $('.searchbar').width();
    var searchWidth;
    var searchWidthFocus;
    if ($('body').hasClass('ios')) {
        searchWidth = searchbarWidth - 184;
        searchWidthFocus = searchbarWidth - 50;
    }else {
        searchWidth = searchbarWidth - 122;
        searchWidthFocus = searchbarWidth - 50;
    };
    $('.search').css('width',searchWidth+'px');
    $('.search').focus(function() {
        $(this).css('width',searchWidthFocus+'px');
        $('.template-selector').hide();
        $('.search-filter.achieveFilter').hide();
        $('.search-cancel').show();
    });
    $('.search').blur(function() {
        $(this).css('width',searchWidth+'px');
        $('.template-selector').show();
        $('.search-filter.achieveFilter').show();
        $('.search-cancel').hide();
    });
};

marketstar.MissionsMain.prototype.changeLayout = function(e) {
    var template = e.target.id;
    this.localStorage.save('missionTemplate', template, this, this.updateModelFromLocalStorage);
    marketstar.missionsMain.filterMissions();
    //marketstar.missionsMain.showChallenges();
};

marketstar.MissionsMain.prototype.filterMissions = function(e) {

    if(e && !e.target.type && e.target.type != 'checkbox')
        return;

    var arr = this.model.get('achievements');
    if($('.achieveSearch').val().trim().length > 0){
        var criteria = $('.achieveSearch').val();

        arr = $.grep(arr, function(o) {
            return (o.Name.toLowerCase().indexOf(criteria.toLowerCase()) > -1 || o.Points.toString().indexOf(criteria.toString()) > -1 || (o.Description != null ? o.Description.toLowerCase().indexOf(criteria.toLowerCase()) > -1 : null));
            //|| (o.Training ? "training" : "mission").indexOf(criteria.toLowerCase()) > -1) ;
        });
    }

    if(e && e.target.id == 'tagAll'){
        if(e.target.checked){
            $('#tags input').prop('checked', true);
        }else{
            $('#tags input').prop('checked', false);
        }
    }

    if(e && e.target.id == 'pointAll'){
        if(e.target.checked){
            $('#points input').prop('checked', true);
        }else{
            $('#points input').prop('checked', false);
        }
    }
    if(e && e.target.id == 'categoryAll'){
        if(e.target.checked){
            $('#categories input').prop('checked', true);
        }else{
            $('#categories input').prop('checked', false);
        }
    }

    var selected = $('.ios-switch').map(function(){if(this.checked && this.id.indexOf('All') == -1) {return {"group":$(this).closest('.filters').attr('id'), "name": this.id };}});

    /*get all filters that have been turned off*/
    var points = $('.ios-switch').map(function(){if(!this.checked && this.id.indexOf('All') == -1 && $(this).closest('.filters').attr('id') == 'points') {return {"name": this.id };}});
    var tags = $('.ios-switch').map(function(){if(!this.checked && this.id.indexOf('All') == -1 && $(this).closest('.filters').attr('id') == 'tags') {return {"name": this.id };}});
    var categories = $('.ios-switch').map(function(){if(!this.checked && this.id.indexOf('All') == -1 && $(this).closest('.filters').attr('id') == 'categories') {return {"name": this.id };}});
    var types = $('.ios-switch').map(function(){if(!this.checked && this.id.indexOf('All') == -1 && $(this).closest('.filters').attr('id') == 'types') {return {"name": this.id };}});
    var statuses = $('.ios-switch').map(function(){if(!this.checked && this.id.indexOf('All') == -1 && $(this).closest('.filters').attr('id') == 'statuses') {return {"name": this.id };}});



    if(statuses.length > 0) {
        $.each(statuses, function(i, status) {arr = $.grep(arr, function (o) {return status.name == 'complete' ? !o.Completed : o.Completed;});});
    }

    if(types.length > 0) {
        $.each(types, function(i, type) {arr = $.grep(arr, function (o) {return type.name == 'training' ? !o.Training : o.Training;});});

    }

    if(tags.length > 0) {
        $.each(tags, function(i, tag){arr = $.grep(arr, function(o){return $.inArray(tag.name, o.Tags.length == 0 ? ["None"] : o.Tags.split(",")) == -1;})});
        $('#tagAll').prop('checked', false);
    }
    else{
        $('#tagAll').prop('checked', true);
    }

    if(points.length > 0) {
        $.each(points, function(i, point){arr = $.grep(arr, function(o){return o.Points != point.name;})});
        $('#pointAll').prop('checked', false);
    }else{
        $('#pointAll').prop('checked', true);
    }

    if(categories.length > 0) {
        $.each(categories, function(i, category){arr = $.grep(arr, function(o){if(o.Training.Category) {return o.Training.Category.Name != category.name;}else {return this;}})});
        $('#categoryAll').prop('checked', false);
    }else{
        $('#categoryAll').prop('checked', true);
    }

    this.showChallenges(arr);

};


marketstar.MissionsMain.prototype.searchMissions = function(e) {

    var arr = this.model.get('achievements');
    var criteria = e.target.value;

    arr = $.grep(arr, function(o) {
        return (o.Name.toLowerCase().indexOf(criteria.toLowerCase()) > -1 || o.Points.toString().indexOf(criteria.toString()) > -1 || o.Description.toLowerCase().indexOf(criteria.toLowerCase()) > -1);
                //|| (o.Training ? "training" : "mission").indexOf(criteria.toLowerCase()) > -1) ;
    });

    this.filterMissions();
    //this.showChallenges(arr);

};

/**
 * get Challenges from the MarketStar Api
 */
marketstar.MissionsMain.prototype.getChallenges = function() {
	var urlParams = {
		urlPrefix: this.marketstarApiUrlPrefix()
		,urlPath: this.marketstarApiAchievement
	};

	this.marketstarApiAjaxCall({
		url:       this.buildUrl(urlParams)
		,type:      'GET'
		,modelName: 'achievements'
		,success:   $.proxy(this.getChallengesSuccess, this)
		,fail:      $.proxy(this.getChallengesFail, this)
	});
};

/**
 * getChallenges success callback
 */
marketstar.MissionsMain.prototype.getChallengesSuccess = function() {
    this.localStorage.save('achievements', this.model.get('achievements'), this, this.updateModelFromLocalStorage);
    this.setFilters();
	this.filterMissions();
	//this.showChallenges();
};

/**
 * getChallenges failure callback
 */
marketstar.MissionsMain.prototype.getChallengesFail = function() {
	// TODO: on fail?
};

marketstar.MissionsMain.prototype.setFilters = function(){
    var achievements = this.model.get('achievements');

    var categories = [];
    var deduped = [];
    var trainings = $.grep(achievements, function(e){ if(e.Training) return e;});
    $.each(trainings, function(i, e) {
        if(e.Training.Category != null) {
            if ($.inArray(e.Training.Category.Name, deduped) == -1) {
                deduped.push(e.Training.Category.Name);
                categories.push({"name": e.Training.Category.Name});
            }
        }
    });

    $('#categories').html(_.template(this.model.getPath('templates.category'), {categories:categories}));

    var points = [];
    deduped = [];
    $.each(achievements, function(i, e) {
        if ($.inArray(e.Points, deduped) == -1){
            deduped.push(e.Points);
            points.push({"points":e.Points});
        }
    });
    points.sort(function(a, b){return a.points - b.points});
    $('#points').html(_.template(this.model.getPath('templates.point'), {points:points}));

    var tags = [];
    deduped = [];
    $.each(achievements, function(i, e) {
        $.each(e.Tags.split(","), function(ind, ex)
        {
            ex = ex.trim().length == 0 ? "None" : ex;
            if ($.inArray(ex, deduped) == -1){
                deduped.push(ex);
                tags.push({"name":ex});
            }
        });
    });

    $('#tags').html(_.template(this.model.getPath('templates.tag'), {tags:tags}));

};

/**
 * render challenges into list
 */
marketstar.MissionsMain.prototype.showChallenges = function(achieves) {
	var template
		,data
		,child
        ,templ
		,achievements = achieves || this.model.get('achievements')
		,viewCompleted = this.model.get('viewCompleted')
		,$missionList = this.$('.mission-list')
        ,$renderTarget
		,that = this;


    $missionList.empty();
    $('.mission-list').remove();
	$('.training-list').remove();

    template = this.localStorage.readFromLocalStorage("missionTemplate");

    if(!template || template == 'grid') {
        this.$('.padding').append('<div class="mission-list"><div class="mission-group"><ul></ul></div></div>');
        $renderTarget = this.$('div.mission-group ul');
        templ = 'templates.mission';
        //$('.template-selector').css('background-image', "url(img/icons/icon_list.png)");
        $('.template-selector').attr('id', "list");
    }
    else {
        this.$('.padding').append('<ul class="training-list"></ul>');
        $renderTarget = this.$('ul.training-list');
        templ = 'templates.training';
        //$('.template-selector').css('background-image', "url(img/icons/icon_grid.png)");
        $('.template-selector').attr('id', "grid");
    }

    $.each(achievements, function(i, achievement){
        data = $.extend({renderTarget: $renderTarget }, achievement); //$.extend({ renderTarget: that.$('div.mission-group ul') }, achievement);
        template = $(_.template(that.model.getPath(templ), achievement));
        template.find('.mission-detail').hide();
        child = new marketstar.Mission(template, data);
        that.addChild(child);
    });


};