_.namespace('marketstar');

/**
 * Constructor
 *
 * @class ProductDetailMain
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.ProductDetailMain = function(el, data) {
    $.extend(marketstar.ProductDetailMain.prototype, Constants);
    $.extend(marketstar.ProductDetailMain.prototype, UtilMixins);
    $.extend(marketstar.ProductDetailMain.prototype, AjaxMixins);
    $.extend(marketstar.ProductDetailMain.prototype, PageMixins);
    this.construct(el, data);
};

marketstar.ProductDetailMain.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.ProductDetailMain.prototype.init = function() {
    this.localStorage = new marketstar.LocalStorageObserverProxy();

    this.parseQueryParametersIntoModel();
    if( !this.model.getPath('queryParams.productId') ) {
        this.goBack();
    }

    this.productId = this.model.getPath('queryParams.productId');
    this.localStorage.registerObserver('products', this, this.updateModelFromLocalStorage);
    this.$('a.back-arrow').on('click', function(e){
        location.href = "productsMain.html";
    });

    this.model.set(
        'templatePaths'
        ,{
            navigation:             '_navigation.html'
            , userBar:              '_userBar.html'
            , socialLinks:          '_socialLinks.html'
            , appVersion: '_appVersion.html'
            , productDetailView:    '_productDetailView.html'
        }
    );

    this.loadIntoModelFromLocalStorage(
        this.userDataKey
        ,$.proxy(this.loadTemplates, this)
        ,$.proxy(this.redirectToIndex, this)
    );
};


/**
 * onTemplatesReady event handler
 *
 * @method onTemplatesReady
 */
marketstar.ProductDetailMain.prototype.onTemplatesReady = function() {
    this.loadIntoModelFromLocalStorage(
        'products'
        ,$.proxy(this.renderProductDetail, this)
        //,this.goBack   TODO What if there is an error? CMS
    );
}


/**
 * Adds the product information to the model.
 *
 * @method getProductDetailSuccess
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.ProductDetailMain.prototype.renderProductDetail = function(requestProps, xhr) {
    var product = this.model.getArrayObject('Id', this.productId, 'products');
    this.model.setPath('product', product);

    var data = $.extend(
        product
        ,{
            renderTarget: this.$('.product-detail-container')
        }
    );

    var template = $(_.template(this.model.getPath('templates.productDetailView'), product));
    $(template).find('.rating-average').addClass(this.convertScoreToClassName(this.normalizeScore(product.Rating.Average)));

    var child = new marketstar.ProductDetailView(
        template
        ,data
    );
    this.addChild(child);
    if(!this.localStorage.readFromLocalStorage('permissions').ProductRatingEnabled){
        $('.rating-info').hide();
        $('.my-rating').hide();
    } else {
        this.renderProductRatings();
    }

};


marketstar.ProductDetailMain.prototype.renderProductRatings = function() {
    this.$('.rating-average').addClass(this.model.getPath('product.Rating.Average'));
    this.$('.my-rating .rating-stars').addClass(
        this.convertScoreToClassName(
            this.normalizeScore(
                this.model.getPath('product.Rating.Mine')
            )
        )
    );
    this.$('a.hit-area').on('click', $.proxy(this.saveProductRating, this));
};


marketstar.ProductDetailMain.prototype.saveProductRating = function(e) {
    e.preventDefault();
    var $x = $(e.currentTarget);

    var  i
        ,classes = ['one', 'two', 'three', 'four', 'five'];

    for(i=0; i<classes.length; i++) {
        $x.parent().removeClass(classes[i]);
        if($x.hasClass(classes[i]) ) {
            if( !$x.parent().hasClass(classes[i]) ) {
                $x.parent().addClass(classes[i]);

                var rating = this.wordToDigit(classes[i]);
                this.model.setPath('savedProductRating', rating);
                this.saveProductRatingToServer(rating);
            }
        }
    }
};

marketstar.ProductDetailMain.prototype.saveProductRatingToServer = function(rating) {
    var urlParams = {
        urlPrefix:  this.marketstarApiUrlPrefix()
        ,urlPath:   'products/' + this.productId + '/ratings'
        ,lang:      this.marketstarDefaultLang
    };

    this.marketstarApiAjaxCall({
        url:       this.buildUrl(urlParams)
        ,type:      'POST'
        ,body:      {Value: rating}
        ,modelName: 'productRatingPostResult'
        ,success:   $.proxy(this.saveProductRatingSuccess, this)
        ,fail:      $.proxy(this.fail, this)
    });
};


marketstar.ProductDetailMain.prototype.saveProductRatingSuccess = function(requestProps, xhr) {
    // Need to update the local store. CMS
    var localStore = this.localStorage.readFromLocalStorage('products');
    for(var l = 0; l < localStore.length; l++) {
        if(localStore[l].Id == this.productId) {
            var postResultRating = this.model.get('productRatingPostResult');
            localStore[l].Rating.Mine = postResultRating.Mine;
            localStore[l].Rating.Average = postResultRating.Average;
            localStore[l].Rating.Count = postResultRating.Count;
            this.localStorage.save('products', localStore, $(this).parent(), null, null);
            break;
        }
    }
};
