_.namespace('marketstar');

/**
 * Constructor
 * 
 * @class ProductDetailView
 * @constructor
 * @extends marketstar.DataView
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.ProductDetailView = function(el, data) {
    $.extend(marketstar.ProductDetailView.prototype, Constants);
    $.extend(marketstar.ProductDetailView.prototype, AjaxMixins);
    $.extend(marketstar.ProductDetailView.prototype, UtilMixins);
    $.extend(marketstar.ProductDetailView.prototype, PageMixins);
    this.construct(el, data);
};

marketstar.ProductDetailView.prototype = Object.create(marketstar.DataView.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.ProductDetailView.prototype.init = function() {
    this.localStorage = new marketstar.LocalStorageObserverProxy();
    this.trainingsLoadAttempts = 0;

    this.model.set(
        'templatePaths'
        ,{
            productPhotoView:   '_productPhotoView.html'
            ,productTrainingView: '_productTrainingView.html'
            ,productSpecificationGroupView: '_productSpecificationGroupView.html'
            ,productSpecificationView: '_productSpecificationView.html'
        }
    );

    this.loadIntoModelFromLocalStorage(
        this.userDataKey
        ,$.proxy(this.loadTemplates, this)
        ,$.proxy(this.redirectToIndex, this)
    );
};


/**
 * Render the page
 *
 * @method templatesReady
 */
marketstar.ProductDetailView.prototype.templatesReady = function() {
    this.renderProductPhotos();
    this.renderProductTrainings();
    this.renderProductSpecificationGroups();
}

/**
 * Render the product photos.
 * @method renderPhotos
 */
marketstar.ProductDetailView.prototype.renderProductPhotos = function() {
    var photos = this.model.get("Photos");
    if( photos && photos.length > 0 ) {
        var i
            ,photo
            ,renderTarget = $('.photo-list')
            ,template
            ,data
            ,child;

        renderTarget.html('');
        for(i = 0; i < photos.length; i++) {
            photo = photos[i];

            template = $(_.template( this.model.getPath('templates.productPhotoView'), photo ));
            $(template).find('a').attr('href','productPhotoSlider.html?productId=' + this.model.get('Id') + '&imageId=' + i);

            data = {
                renderTarget: renderTarget
            };
            child = new marketstar.ProductPhotoView(
                template
                ,data
            );
            this.addChild(child);
        }
    }
}

/**
 * Load the trainings. Yep, this is duplicated from the trainingsMain.js file. That's because SolutionSet's code does not provide an easy way to
 * to run methods in a class statically. So, when I attempted to call this method in trainingsMain.js after instantiating it, it caused all sorts
 * of problems with missing contexts and thus, null variable issues. So, I've duplicated it here and will attempt to refactor it when we refactor
 * SolutionSet's code. CMS
 *
 * @method getTrainingsForType
 * @async
 * @param typeId {Number}
 */
marketstar.ProductDetailView.prototype.getTrainings = function() {
    this.trainingsLoadAttempts++;
    var urlParams = {
        urlPrefix:  this.marketstarApiUrlPrefix()
        ,urlPath:   'trainings?lang={lang}'
        ,lang:      this.marketstarDefaultLang
    };

    this.marketstarApiAjaxCall({
        url:       this.buildUrl(urlParams)
        ,type:      'GET'
        ,modelName: 'trainings'
        ,success:   $.proxy(this.getTrainingsSuccess, this)
        ,fail:      $.proxy(this.getTrainingsFail, this)
    });
};

/**
 * Success method for getTrainings(). See note in getTrainings(). CMS
 *
 * @method getTrainingsSuccess
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.ProductDetailView.prototype.getTrainingsSuccess = function(requestProps, xhr) {
    if(xhr) {
        this.localStorage.save('trainings', this.model.getPath('trainings'), this, null, null);
        this.renderProductTrainings();
    }
};

/**
 * Fail callback for getTrainings(). See note in getTrainings(). CMS   s
 *
 * @method getTrainingsFail
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.ProductDetailView.prototype.getTrainingsFail = function(requestProps, xhr) {
    // TODO: Fail
    console.log(xhr);
};

/**
 * Render the product trainings.
 * @method renderProductTrainings
 */
marketstar.ProductDetailView.prototype.renderProductTrainings = function() {
    var localStoreTrainings = this.localStorage.read('trainings'); // Retrieving from localstorage because the local product trainings does not contain the usercompleted information. CMS

    // If there are no trainings in localStorage, load them from the web service. CMS
    if(localStoreTrainings == null && this.trainingsLoadAttempts <= 3) {
        this.getTrainings();
        return;
    }

    var trainings = this.model.get("Trainings");
    if( trainings && trainings.length > 0 ) {
        var i
            ,l
            ,training
            ,renderTarget = $('.training-modules')
            ,template
            ,data
            ,child;

        renderTarget.html('');
        for(i = 0; i < trainings.length; i++) {
            training = trainings[i];

            // if there is a local storage object for the training, load it from there. CMS
            for(l = 0; l < localStoreTrainings.length; l++) {
                if(localStoreTrainings[l].Id == training.Id) {
                    training = localStoreTrainings[l];
                    break;
                }
            }

            template = $(_.template( this.model.getPath('templates.productTrainingView'), training ));
            $(template).find('a').attr('data-href','trainingDetail.html?trainingId=' + training.Id);
            $(template).on('click', $.proxy(this.redirectToTraining, this));

            data = {
                renderTarget: renderTarget
            };
            child = new marketstar.ProductTrainingView(
                template
                , $.extend(data, training)
            );
            this.addChild(child);
        }
    }
}


marketstar.ProductDetailView.prototype.redirectToTraining = function(e) {
    location.href = decodeURIComponent($(e.currentTarget).find('a').data('href'));
}


/**
 * Render the product specifications.
 * @method renderSpecifications
 */
marketstar.ProductDetailView.prototype.renderProductSpecificationGroups = function() {
    var specificationGroups = this.model.get("Specifications");
    if( specificationGroups && specificationGroups.length > 0 ) {
        var i
            ,renderTarget = $('.product-specifications');

        renderTarget.html('');
        for(i = 0; i < specificationGroups.length; i++) {
            var specificationGroup
                ,template
                ,data
                ,child;
            specificationGroup = specificationGroups[i];
            var specGroupTemplate = $(_.template( this.model.getPath('templates.productSpecificationGroupView'), specificationGroup ));
            var specTemplate = $(this.model.getPath('templates.productSpecificationView'));

            var specVal = specificationGroup.SpecificationValues;
            for(var s = 0; s < specVal.length; s++) {
                var clonedTemplate = specTemplate.clone();
                $(clonedTemplate).find('.name').html(specVal[s].Name);
                $(clonedTemplate).find('.value').html(specVal[s].Value);
                $(specGroupTemplate).find('.items').append(clonedTemplate);
            }
            data = {
                renderTarget: renderTarget
            };
            child = new marketstar.ProductSpecificationGroupView(
                specGroupTemplate
                ,data
            );
            this.addChild(child);
        }
    }
}


/**
 * Post render() callback.  Contract requirement from DataView.
 * Put anything here that is dependent on the existence of elements in the DOM
 * 
 * @method postRender
 */
marketstar.ProductDetailView.prototype.postRender = function() {
};
