_.namespace('marketstar');

/**
 * Constructor
 *
 * @class ProductPhotoView
 * @constructor
 * @extends marketstar.DataView
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.ProductPhotoSlider = function(el, data) {
    $.extend(marketstar.ProductPhotoSlider.prototype, Constants);
    $.extend(marketstar.ProductPhotoSlider.prototype, AjaxMixins);
    $.extend(marketstar.ProductPhotoSlider.prototype, UtilMixins);
    $.extend(marketstar.ProductPhotoSlider.prototype, PageMixins);
    this.construct(el, data);
};

marketstar.ProductPhotoSlider.prototype = Object.create(marketstar.DataView.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.ProductPhotoSlider.prototype.init = function() {
    this.localStorage = new marketstar.LocalStorageObserverProxy();

    this.parseQueryParametersIntoModel();
    if( !this.model.getPath('queryParams.productId') ) {
        location.href = 'productMain.html';
    }

    this.productId = this.model.getPath('queryParams.productId');
    this.localStorage.registerObserver('products', this, this.updateModelFromLocalStorage);
    this.$('a.back-arrow').on('click', this.goBack);

    this.model.set(
        'templatePaths'
        ,{
            navigation: '_navigation.html'
            , userBar: '_userBar.html'
            , socialLinks: '_socialLinks.html'
            , appVersion: '_appVersion.html'
            , productPhotoSliderView:   '_productPhotoSliderView.html'
        }
    );

    this.loadIntoModelFromLocalStorage(
        this.userDataKey
        ,$.proxy(this.loadTemplates, this)
        ,$.proxy(this.redirectToIndex, this)
    );
};

/**
 * Render the page
 *
 * @method templatesReady
 */
marketstar.ProductPhotoSlider.prototype.templatesReady = function() {
    this.loadIntoModelFromLocalStorage(
        'products'
        ,$.proxy(this.renderProductPhotoSlider, this)
        //,this.goBack   TODO What if there is an error? CMS
    );
};

/**
 * Render the photos
 *
 * @method renderProductPhotoSlider
 */
marketstar.ProductPhotoSlider.prototype.renderProductPhotoSlider = function() {
    this.renderNavigation();

    var product = this.model.getArrayObject('Id', this.productId, 'products')
        ,renderTarget = $('.slides');

    $('.page-title').html(product.Name + ' Photos');

    this.productPhotos = product.Photos;

    for(var p = 0; p < this.productPhotos.length; p++) {
        var template = _.template(this.model.getPath('templates.productPhotoSliderView'), this.productPhotos[p]);

        var data = {
            renderTarget: renderTarget
        };

        var child = new marketstar.ProductPhotoSliderView(
            template
            ,data
        );
        this.addChild(child);
    }
    $.proxy(this.renderFlexSlider, this)();
};

/**
 * Post render() callback.  Contract requirement from DataView.
 * Put anything here that is dependent on the existence of elements in the DOM
 *
 * @method postRender
 */
marketstar.ProductPhotoSlider.prototype.postRender = function() {

};

/**
 * Render Flex Slider
 *
 * @method renderFlexSlider
 */
marketstar.ProductPhotoSlider.prototype.renderFlexSlider = function() {
    //var startImg = 2;
    //$.event.special.swipe.horizontalDistanceThreshold = 200;
    var setSlider = function() {
        $('.flexslider').flexslider({
            animation: 'slide'
            ,animationLoop: false
            ,startAt: startImg
            ,slideshow: false
            ,itemMargin: 40
            ,start: $.proxy(function(slider) {
                var $slides = $('.slides li');
                $slides.on('swipeleft', function(event){
                    slider.flexAnimate(slider.getTarget("next"));
                });
                $slides.on('swiperight', function(event){
                    slider.flexAnimate(slider.getTarget("previous"));
                });
                var $topBar = $('.top-bar');
                $slides.on('click', function(event){

                    if( $topBar.is(':animated') ) {
                    } else {
                        if( $topBar.is(':visible') ) {
                            $topBar.animate({top:'-200px'}, 600, function() {$('.top-bar').css('display','none');});
                        }
                        else {
                            $topBar.css('display','inline').animate({top:'0'}, 600, function() {setTimeout(slideTopBar,4000);});
                        }
                    }
                });
                var slideTopBar = function() {
                    $('.top-bar').animate({top:'-200px'}, 600, function() {$('.top-bar').css('display','none');});
                }
                setTimeout(slideTopBar,2000);

                this.repositionFlexSliderImages();
            }, this)
        })
    }
    setTimeout($.proxy(setSlider, this),1000);

    var t, w = 0, h = 0, $win;
    $(window).on('resize', $.proxy(function() {
        // If the window size has changed, resize the flexslider. CMS

        $win = $(window);
        if($win.height() != h || $win.width() != w) {
            h = $win.height();
            w = $win.width();

            if(t) {
                clearTimeout(t);
            }
            // This prevents the reposition function being called about 1000 times during the resize. CMS
            t = setTimeout(this.repositionFlexSliderImages, 500);
        }
    }, this));
};

/**
 * Reposition Flex Slider Images
 *
 * @method repositionFlexSliderImages
 */
marketstar.ProductPhotoSlider.prototype.repositionFlexSliderImages = function() {
    $('.flexslider').data('flexslider').resize()
    // Update the heights of the viewport to take up the screen and vertically align the images to middle. CMS

    var $flexSliderParent = $('.container.slider');
    $('.flexslider').height($flexSliderParent.height());

    $('.flexslider .slides li').height($flexSliderParent.height());

    var $img = $('.flexslider .slides img');
    $img.each(function(idx, val){
        var $v = $(val);
        var h = Math.floor(($v.parent().height() - $v.height()) / 2);
        $v.css('margin-top', h + 'px');
    });
};
