_.namespace('marketstar');

/**
 * Constructor
 *
 * @class ProductsMain
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.ProductsMain = function(el, data) {
    $.extend(marketstar.ProductsMain.prototype, Constants);
    $.extend(marketstar.ProductsMain.prototype, UtilMixins);
    $.extend(marketstar.ProductsMain.prototype, AjaxMixins);
    $.extend(marketstar.ProductsMain.prototype, PageMixins);
    this.construct(el, data);
};

marketstar.ProductsMain.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.ProductsMain.prototype.init = function() {
    this.localStorage = new marketstar.LocalStorageObserverProxy();

    this.model.set(
        'templatePaths'
        ,{
            navigation: '_navigation.html'
            , userBar: '_userBar.html'
            , socialLinks: '_socialLinks.html'
            , appVersion: '_appVersion.html'
            , product:   '_product.html'
        }
    );

    this.loadIntoModelFromLocalStorage(
         this.userDataKey
        ,$.proxy(this.loadTemplates, this)
        ,$.proxy(this.redirectToIndex, this)
    );
};

/**
 * Render the page
 *
 * @method templatesReady
 */
marketstar.ProductsMain.prototype.onTemplatesReady = function() {
    //this.getCategories();

};

/**
 * Load the categories
 *
 * @method getCategories
 * @async
 */
marketstar.ProductsMain.prototype.getCategories = function() {
    var urlParams = {
         urlPrefix: this.marketstarApiUrlPrefix()
        ,urlPath:   this.marketstarApiProductCategoriesPath
    };

    this.marketstarApiAjaxCall({
        url:       this.buildUrl(urlParams)
        , type:      'GET'
        , modelName: this.productCategoriesModelKey
        , success:   $.proxy(this.getCategoriesSuccess, this)
        , fail:      $.proxy(this.getCategoriesFail, this)
    });
};

/**
 * Adds the categories as <option>'s to the <select>.
 * Success callback for getCategories
 *
 * @method getTrainingTypesSucess
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.ProductsMain.prototype.getCategoriesSuccess = function(requestProps, xhr) {
    var modelName = this.productCategoriesModelKey
        ,categories = []
        ,props = {
             selector:        '.selector-container .selector'
            ,coll:            this.model.get(modelName)
            ,idField:         'Id'
            ,nameField:       'Name'
            ,onChangeHandler: $.proxy(this.getProductsByCategoryHandler, this)
            ,initialLoadFun:  $.proxy(this.getProductsByCategory, this)
            ,defaultElement:  {
                                 Id:   0
                                ,Name: this.allText
                              }
        };

    $.each(props.coll, function(i, category){
        if (category.ParentCategoryId) {
            categories.push(category);
        }
    });

    props.coll = categories;

    // stuff in local storage, register an observer
    this.localStorage.save(modelName, this.model.get(modelName), this, this.updateModelFromLocalStorage, null);

    if(this.model.data.productCategories.length > 1)
        this.$('.selector-container').show();

    this.updateUl(props);

};

/**
 * Failure callback for getCategories
 *
 * @method getCategoriesFail
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.ProductsMain.prototype.getCategoriesFail = function(requestProps, xhr) {
    // TODO: handle failure
};

/**
 * Event handler for category <select>.onChange()
 *
 * @method getProductsByCategoryHandler
 * @event change
 * @param e {Object}
 */
marketstar.ProductsMain.prototype.getProductsByCategoryHandler = function(e) {
    this.getProductsByCategory(e.target.options[e.target.options.selectedIndex].value);
};

/**
 * Load products by category
 *
 * @method getProductsByCategory
 * @async
 * @param categoryId {Number}
 */
marketstar.ProductsMain.prototype.getProductsByCategory = function(categoryId) {
    if(this.model.get('category' + categoryId)) {
        this.getCategoriesSuccess({modelName: 'category' + categoryId}, null);
    }

    var urlParams = {
         urlPrefix:  this.marketstarApiUrlPrefix()
        ,urlPath:    this.marketstarApiProductsByCategoryPath
        ,categoryId: categoryId
    };

    if(categoryId == 0) {
       urlParams = $.extend( urlParams, { urlPath: this.marketstarApiProductsPath } );
    }

    this.marketstarApiAjaxCall({
        url:       this.buildUrl(urlParams)
        , type:      'GET'
        , showLoader: true
        , modelName: 'products' + categoryId
        , success:   $.proxy(this.getProductsByCategorySuccess, this)
        , fail:      $.proxy(this.getProductsByCategoryFail, this)
    });
};

/**
 * Render <li>'s into the <ul> containing stream activities using sudo template 'templates.productsMainView'
 * Success callback for getProductsByCategory
 *
 * @method getProductsByCategorySuccess
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.ProductsMain.prototype.getProductsByCategorySuccess = function(requestProps, xhr) {
    var  i
        ,product
        ,products = this.model.get(requestProps.modelName)
        ,template
        ,data
        ,child;

    this.$('ul.product-list').empty();

    for(i=0; i<products.length; i++) {
        product = products[i];

        template = _.template(this.model.getPath('templates.product'), product);

       	data = {
			renderTarget: this.$('ul.product-list')
    	};
        child = new marketstar.Product(
                                            template
                                            ,$.extend(data, product)
                                        );
        this.addChild(child);
    }

    if(!this.localStorage.readFromLocalStorage('permissions').ProductRatingEnabled){
        $('.rating-info').hide();
    }


    this.model.setPath('products', products);

    if(xhr) {
        this.localStorage.save('products', this.model.get('products'), this, this.updateModelFromLocalStorage, null);
    }
};

/**
 * Failure callback for getProductsByCategoryFail
 *
 * @method getProductsByCategoryFail
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.ProductsMain.prototype.getProductsByCategoryFail = function(requestProps, xhr) {
    // TODO: handle failure
};