(function() {
    var root = this;
    var Connect = root.Connect = {};

    /******* Properties *******/
    Connect.VERSION = '1.0.0';
    Connect.ApiUrl = null;
    Connect.ApiKey = null;
    Connect.ApiSecretKey = null;
    Connect.Timestamp = null;
    Connect.Signature = null;
    Connect.UserId = null;
    Connect.$ = root.jQuery || root.Zepto || root.ender || root.$;

    var routes = {
        Achievements: { route: ':instanceUri/api/achievements/:id', accepts: ['get'] }
        ,AchievementRatings: { route: ':instanceUri/api/achievements/:id/ratings', accepts: ['get', 'post', 'put', 'delete'] }
        ,Certifications: { route: ':instanceUri/api/certifications', accepts: ['get'] }
        ,CertificationCategories: { route: ':instanceUri/api/certifications/categories', accepts: ['get'] }
        ,GiftPoints: { route: ':instanceUri/api/giftpoints', accepts: ['post'] }
        ,GiftPointsMaxAllowed: { route: ':instanceUri/api/giftpoints/maxallowed', accepts: ['get'] }
        ,ProductCategories: { route: ':instanceUri/api/products/categories', accepts: ['get'] }
        ,Product: { route: ':instanceUri/api/products/:id', accepts: ['get'] }
        ,Products: { route: ':instanceUri/api/products', accepts: ['get'] }
        ,ProductRatings: { route: ':instanceUri/api/products/:id/ratings', accepts: ['get', 'post'] }
        ,ProductSearch: { route: ':instanceUri/api/products/search', accepts: ['get'] }
        ,ProductSpecifications: { route: ':instanceUri/api/productspecifications', accepts: ['get'] }
        ,UserPromotions: { route: ':instanceUri/api/user/promotions', accepts: ['get'] }
        ,Rep: { route: ':instanceUri/api/user/rep', accepts: ['get'] }
        ,Stores: { route: ':instanceUri/api/stores', accepts: ['get'] }
        ,StoreCities: { route: ':instanceUri/api/stores/cities', accepts: ['get'] }
        ,StoresStates: { route: ':instanceUri/api/stores/states', accepts: ['get'] }
        ,StoreSources: { route: ':instanceUri/api/storesources', accepts: ['get'] }
        ,StoreConfirm: { route: ':instanceUri/api/storeconfirm', accepts: ['get'] }
        ,Training: { route: ':instanceUri/api/trainings/:id', accepts: ['get'] }
        ,Trainings: { route: ':instanceUri/api/trainings/', accepts: ['get'] }
        ,TrainingCategories: { route: ':instanceUri/api/trainings/categories', accepts: ['get'] }
        ,TrainingModule: { route: ':instanceUri/api/trainings/:trainingId/modules/:id', accepts: ['get'] }
        ,UserAnswers: { route: ':instanceUri/api/trainings/:trainingId/modules/:trainingModuleId/useranswers/:id', accepts: ['get']}
        ,UserInfo: { route: ':instanceUri/api/user/info', accepts: ['get'] }
        ,UserReferralCodes: {route: ':instanceUri/api/user/ReferralCodes', accepts: ['get'] }
        ,UserStore: { route: ':instanceUri/api/user/store', accepts: ['get'] }
        ,UserCertifications: { route: 'instanceUri/api/user/certifications', accepts: ['get']}
        ,DailyQuestions: { route: ':instanceUri/api/dailyquestions', accepts: ['get', 'post', 'put'] }
        ,DailyQuestionAnswer: { route: ':instanceUri/api/dailyquestionanswer', accepts: ['post'] }
        ,UserPointsGifted: { route: ':instanceUri/api/user/pointsgifted', accepts: ['get'] }
		,UserReferral: { route: ':instanceUri/api/userreferrals', accepts: ['post'] }
        ,UserReferrals: { route: ':instanceUri/api/user/Referrals', accepts: ['get'] }
		,UserPermissions: { route: ':instanceUri/api/user/permissions', accepts: ['get'] }
        ,UserPromotionRegistrations: { route: ':instanceUri/api/user/promotionregistrations', accepts: ['get'] }
        ,UserConfirmStore: { route: ':instanceUri/api/user/confirmstore', accepts: ['post'] }
        ,UserRemoveAccess: { route: ':instanceUri/api/user/removeaccess', accepts: ['post'] }
        ,UserStoreChange: { route: ':instanceUri/api/user/storechange', accepts: ['post'] }
        ,UserChangeStoreNotFound: { route: ':instanceUri/api/user/changestorenotfound', accepts: ['post'] }
        ,Promotions: { route: ':instanceUri/api/promotions/:id', accepts: ['get'] }
        ,PromotionRegistrations: { route: ':instanceUri/api/promotionregistrations', accepts: ['post'] }
        ,QuickLinks: { route: ':instanceUri/api/quicklinks', accepts: ['get'] }
    };

    var getTimestamp = function() {
        return Math.floor(new Date().getTime() / 1000);
    };

    var getSignature = function(userId, key, secret, timestamp) {
        return CryptoJS.SHA1(userId + key + secret + timestamp).toString(CryptoJS.enc.Hex);
    };

    var isEmpty = function(value) {
        if (value === undefined || value === null || (typeof(value) === 'string' && value.length == 0))
            return true;
        return false;
    };

    var clone = function(obj) {
        return JSON.parse(JSON.stringify(obj));
    }

    var useXDomainRequest = function() {
        return /MSIE ([7-9].\d+);/.test(navigator.userAgent) && window.XDomainRequest;
    }

    var urlAppendObj = function(url, obj) {
        if (obj == null || Object.keys(obj).length == 0) return url;
        url += url.indexOf('?') == -1 ? '?' : '&';
        var i = 0;
        for(var key in obj) {
            url += (i > 0 ? '&' : '') + key + '=' + obj[key];
            i++;
        }
        return url;
    }

    var request = {
        ajax: function(options) {
            var async = options.async != null ? options.async : true
                ,type = options.type || 'get'
                ,url = options.url || ''
                ,onSuccess = options.success || function() {}
                ,onComplete = options.complete || function() {}
                ,onError = options.error || function() {}
                ,headers = options.headers || {}
            body = options.data || {};

            if (type == 'get') {
                url = urlAppendObj(url, body)
            }

            if (useXDomainRequest()) {
                var xdr = new window.XDomainRequest();
                url = urlAppendObj(url, headers);
                xdr.open(type, url, async);
                xdr.onload = function() {
                    var r = xdr.responseText;
                    var d = null; try { d = JSON.parse(r); } catch (e) { d = r; };
                    xdr = undefined;
                    onSuccess(d);
                    onComplete(d);
                };
                xdr.onerror = function() {
                    onError(xdr, xdr.responseText);
                };
                xdr.onprogress = function() {};
                if (options.timeout) {
                    xdr.ontimeout = function() {
                        callback(0, 'timeout');
                    };
                    xdr.timeout = options.timeout;
                }
                xdr.send(JSON.stringify(body));
                return xdr;
            } else {
                var xhr = new XMLHttpRequest();
                xhr.open(type, url, async);
                xhr.timeout = options.timeout || 0;
                xhr.onerror = function() {
                    onError(xhr, xhr.response);
                };
                xhr.onload = function() {
                    if(xhr.readyState === 4) {
                        if (xhr.status >= 200 && xhr.status < 300) {
                            var x = null; try { x = JSON.parse(xhr.response); } catch (e) { x = xhr.response; };
                            onSuccess(x);
                        }
                        else {
                            onError(xhr);
                        }
                        onComplete();
                    }
                };
                xhr.setRequestHeader('Accept', 'application/json');
                xhr.setRequestHeader('Content-Type', 'application/json');
                for (var p in headers) {
                    xhr.setRequestHeader(p, headers[p]);
                }
                xhr.send(JSON.stringify(body));
                return xhr;
            }
        }
        ,send: function(endPoint, type, options, params) {
            var timestamp = Connect.Timestamp || getTimestamp();
            var signature = Connect.Signature || getSignature(Connect.UserId, Connect.ApiKey, Connect.ApiSecretKey, timestamp);
            var defaultHeaders = {
                UserId: Connect.UserId
                ,ApiKey: Connect.ApiKey
                ,Timestamp: timestamp
                ,Signature: signature
            };
            options = options || {};
            options.url = router(endPoint, params);
            options.type = type;
            options.headers = extend(defaultHeaders, options.headers || {});
            request.ajax(options);
        }
    };

    /**
     * router
     * @param route
     * @param params
     * @returns {string}
     */
    var router = function(route, params) {
        var p = { instanceUri: Connect.ApiUrl };
        if (typeof params === "object") p = extend(p, params);
        var namedParam = /(\(\?)?:\w+/g;
        var r = route.replace(namedParam, function(match) {
            var name = match.substring(1, match.length);
            return p[name] || '';
        });
        if (typeof params === "string") {
            if (params.indexOf('?') !== 0) r += '?';
            r += params;
        }
        return r;
    };

    /**
     * extend method for merging objects
     * @type {Function}
     */
    var extend = (Connect.$ !== undefined) ? Connect.$.extend : function(obj) {
        for (var i = 0; i < arguments.length; i++) {
            var source = arguments[i];
            if (source) {
                for (var prop in source) {
                    obj[prop] = source[prop];
                }
            }
        }
        return obj;
    };

    for (var prop in routes) {
        var route = routes[prop];
        var obj = Connect[prop] = function(id) {
            var arg = arguments[0] || {};
            this.id = (typeof arg === 'number' || typeof arg === 'string') ? arg : null;
            this.data = (typeof arg === 'object') ? arg : null;
            return this;
        };
        for(var i = 0; i < route.accepts.length; i++) {
            (function(route, type) {
                obj.prototype[type] = function(options) {
                    var that = this;
                    var opts = isEmpty(options) ? {} : clone(options);
                    opts.success = function(data) {
                        that.data = data;
                        if (options && options.success) options.success(that);
                    }
                    opts.error = options.error || function() {};
                    opts.data = ((options && options.data) ? options.data : that.data) || {};
                    var params = opts.data;
                    if (!isEmpty(that.id)) params.id = that.id;
                    request.send(route, type, opts, params);
                    return that;
                };
            }(route.route, route.accepts[i]));
        };
    };

}).call(this);

function getUnderscore() {
    if (typeof _ == "undefined")
        return $j.getScript('/plugins/connectplus/resources/script/lib/underscore-min.js');
    return null;
};

function getBackbone() {
    if (typeof Backbone == "undefined")
        return $j.getScript('/plugins/connectplus/resources/script/lib/backbone-min.js');
    return null;
};
