_.namespace('marketstar');

/**
 * Constructor
 *
 * @class SalesCaptureMain
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.SalesCaptureMain = function (el, data) {
    $.extend(marketstar.SalesCaptureMain.prototype, Constants);
    $.extend(marketstar.SalesCaptureMain.prototype, UtilMixins);
    $.extend(marketstar.SalesCaptureMain.prototype, AjaxMixins);
    $.extend(marketstar.SalesCaptureMain.prototype, PageMixins);
    $.extend(marketstar.SalesCaptureMain.prototype, SalesCaptureMixins);
    this.construct(el, data);
};

marketstar.SalesCaptureMain.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.SalesCaptureMain.prototype.init = function () {
    this.localStorage = new marketstar.LocalStorageObserverProxy();

    this.model.set(
            'templatePaths'
            , {
                navigation: '_navigation.html',
                userBar: '_userBar.html',
                socialLinks: '_socialLinks.html',
                appVersion: '_appVersion.html',
                promotion: '_salesCapturePromotion.html'
            }
    );

    this.loadIntoModelFromLocalStorage(
            this.userDataKey
            , $.proxy(this.loadTemplates, this)
            , $.proxy(this.redirectToIndex, this)
    );
};

/**
 * Render the page
 *
 * @method templatesReady
 */
marketstar.SalesCaptureMain.prototype.onTemplatesReady = function () {
    this.getPromotions();
};

marketstar.SalesCaptureMain.prototype.getPromotions = function(){
    var urlParams = {
        urlPrefix:  this.marketstarApiUrlPrefix()
        ,urlPath:    'user/promotions?mobile=true'
    };

    this.marketstarApiAjaxCall({
        url:            this.buildUrl(urlParams)
        , type:         'GET'
        , modelName:    'promotions'
        , success:      $.proxy(this.getPromotionsSuccess, this)
        , fail:         $.proxy(this.getPromotionsFail, this)
    });
};

marketstar.SalesCaptureMain.prototype.getPromotionsSuccess = function(requestProps, xhr){
    this.localStorage.save('promotions', this.model.get(requestProps.modelName), this, this.updateModelFromLocalStorage, null);

    var  i
            ,promotions = this.model.get(requestProps.modelName)
            ,template
            ,data
            ,child
            ,callScanner;

    promotions = $.grep(promotions, function(o, i){
        if(o) {
            return o.EnterInMobileApp == true;
        }
    });

    //used to prevent looping on scanner when there is only one promotion (obsolete if allowScan is set to false)
    callScanner = UtilMixins.getQueryParametersByName('callScanner');

    this.$('ul.scannable-promotions-list').empty();

    data = {
        renderTarget: this.$('ul.scannable-promotions-list')
    };

    if(promotions.length == 0)
    {
      callScanner = false;
      promotions[0] = {
         Nopromotions: true
      }
    }

    if(promotions.length == 1 && (callScanner && Config.allowScan))
    {
        //template = _.template(this.model.getPath('templates.promotion'), promotions);

        //child = new marketstar.SalesCapturePromotion(
        //        template
        //        ,$.extend(promotions[0], data)
        //);
        //this.addChild(child);
        $('.promotion').hide();
        //child.showScanner(promotions[0].Id);
        this.showScanner(promotions[0].Id);
    }
    else {
        for (i = 0; i < promotions.length; i++) {
            var promo = promotions[i];

            template = _.template(this.model.getPath('templates.promotion'), promo);

            child = new marketstar.SalesCapturePromotion(
                    template
                    , $.extend(promo, data)
            );

            this.addChild(child);
        }
    }

};

marketstar.SalesCaptureMain.prototype.getPromotionsFail = function(){

};


marketstar.SalesCaptureMain.prototype.onResize = function () {
   // this.hideLoader();
   // this.frame.height(this.frame.contents().height());
   // $(window).scrollTop(0);
};


