_.namespace('marketstar');

/**
 * Constructor
 *
 * @class ProductDetailMain
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.SalesCapturePromotionDetail = function(el, data) {
    $.extend(marketstar.SalesCapturePromotionDetail.prototype, Constants);
    $.extend(marketstar.SalesCapturePromotionDetail.prototype, UtilMixins);
    $.extend(marketstar.SalesCapturePromotionDetail.prototype, AjaxMixins);
    $.extend(marketstar.SalesCapturePromotionDetail.prototype, PageMixins);
    $.extend(marketstar.SalesCapturePromotionDetail.prototype, SalesCaptureMixins);
    this.construct(el, data);
};

marketstar.SalesCapturePromotionDetail.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.SalesCapturePromotionDetail.prototype.init = function() {
    this.localStorage = new marketstar.LocalStorageObserverProxy();

    this.parseQueryParametersIntoModel();
    if( !this.model.getPath('queryParams.promotionId') ) {
        this.goBack();
    }

    this.promotionId = this.model.getPath('queryParams.promotionId');
    //this.localStorage.registerObserver('products', this, this.updateModelFromLocalStorage);
    this.$('a.back-arrow').on('click', function(e){
        location.href = "salesCaptureMain.html";
    });

    this.model.set(
            'templatePaths'
            ,{
                navigation:             '_navigation.html'
                , userBar:              '_userBar.html'
                , socialLinks:          '_socialLinks.html'
                , appVersion:           '_appVersion.html'
                , promotionDetailView:  '_salesCapturePromotionDetail.html'
            }
    );

    this.loadIntoModelFromLocalStorage(
            this.userDataKey
            ,$.proxy(this.loadTemplates, this)
            ,$.proxy(this.redirectToIndex, this)
    );
};


/**
 * onTemplatesReady event handler
 *
 * @method onTemplatesReady
 */
marketstar.SalesCapturePromotionDetail.prototype.onTemplatesReady = function() {
    this.loadIntoModelFromLocalStorage(
            'promotions'
            ,$.proxy(this.renderPromotionDetail, this)
            //,this.goBack   TODO What if there is an error? CMS
    );
}


/**
 * Adds the product information to the model.
 *
 * @method getProductDetailSuccess
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.SalesCapturePromotionDetail.prototype.renderPromotionDetail = function(requestProps, xhr) {
    var promotion = this.model.getArrayObject('Id', this.promotionId, 'promotions');
    this.model.setPath('promotion', promotion);

    var data = $.extend(
            promotion
            ,{
                renderTarget: this.$('.promotion-detail-container')
            }
    );

    var template = $(_.template(this.model.getPath('templates.promotionDetailView'), promotion));

    var child = new marketstar.PromotionDetailView(
            template
            ,data
    );
    this.addChild(child);
};


marketstar.SalesCapturePromotionDetail.prototype.showScanner = function(promotionId, e) {
    this.showScanner(promotionId);
    //this.parent.postPromotion(promotionId, '041710132483');// '052800482623');// '012571000035515');

};

marketstar.SalesCapturePromotionDetail.prototype.showPrompt = function(promotionId, e){
    var code, that = this;

    if(window.device){
        window.navigator.notification.prompt(
                new String(), // message
                function(answer) {
                    if (answer.buttonIndex === 1) {
                        // Submit
                        code = answer.input1;
                        if(code.trim().length != 0)
                            that.postPromotion(promotionId, code);
                    }
                    else {
                        // Exit
                    }
                }, // callback
                "Please enter your code", //title
                ["Submit", "Cancel"], // button titles
                new String() // defaultText
        );
    } else {
        code = prompt("Please enter your code");
        if (code != null) {
            this.postPromotion(promotionId, code)
        }
    }

};

/**
 * Post render() callback.  Contract requirement from DataView.
 * Put anything here that is dependent on the existence of elements in the DOM
 *
 * @method postRender
 */
marketstar.SalesCapturePromotionDetail.prototype.postRender = function() {
};



