_.namespace('marketstar');

/**
 * Constructor
 *
 * @class Shop
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.Shop = function (el, data) {
    $.extend(marketstar.Shop.prototype, Constants);
    $.extend(marketstar.Shop.prototype, UtilMixins);
    $.extend(marketstar.Shop.prototype, AjaxMixins);
    $.extend(marketstar.Shop.prototype, PageMixins);
    this.construct(el, data);
};

marketstar.Shop.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.Shop.prototype.init = function () {
    this.localStorage = new marketstar.LocalStorageObserverProxy();

    this.model.set(
        'templatePaths'
        , {
            navigation: '_navigation.html',
            userBar: '_userBar.html',
            socialLinks: '_socialLinks.html',
            appVersion: '_appVersion.html'
        }
    );

    this.loadIntoModelFromLocalStorage(
        this.userDataKey
        , $.proxy(this.loadTemplates, this)
        , $.proxy(this.redirectToIndex, this)
    );
};


marketstar.Shop.prototype.bindScroller = function () {
    this.iScroll = new iScroll('navscroller', {
        useTransition: false,
        checkDOMChanges: true,


        onBeforeScrollStart: function (e) {
            //e.stopPropagation();
            this.maxScrollY = (parseInt($(window).height()) - parseInt($('.nav-container').height()+10));

            var nodeType = e.explicitOriginalTarget ? e.explicitOriginalTarget.nodeName.toLowerCase():(e.target ? e.target.nodeName.toLowerCase():'');

            if(nodeType !='select' && nodeType !='option' && nodeType !='input' && nodeType!='textarea' ) {
                e.preventDefault();
            }
        }

    });
};


/**
 * Render the page
 *
 * @method templatesReady
 */
marketstar.Shop.prototype.onTemplatesReady = function () {
    this.showLoader();
    $('#loadingShop').show();
    //var url = encodeURIComponent(this.buildMarketStarUrl('shop', null));

    //this.shopWindow = this.openUrlInAppBrowser("shopFrame.html?url=" + url);

    var userId = this.model.getPath(this.userDataKey + '.id');
    this.frame = this.$('iframe');

    var timestamp = Math.floor(new Date().getTime() / 1000);
    var urlParams = {
        urlPrefix: Config.marketstarUrl,
        urlPath: 'shop?UserId={userId}&ApiKey={apiKey}&Timestamp={timestamp}&Signature={signature}',
        userId: userId,
        apiKey: Config.marketstarApiKey,
        timestamp: timestamp,
        signature: this.generateMarketstarApiSignature(timestamp, userId)
    };
    this.frame.attr('src', this.buildUrl(urlParams));
    this.frame.on('load', $.proxy(this.onResize, this));
    this.frame.on('error', $.proxy(this.onIFrameError, this));
    this.frame.show();
    $('.app').show();
}

marketstar.Shop.prototype.onResize = function () {
    this.hideLoader();
    $('#loadingShop').hide();
    this.frame.height(this.frame.contents().height());
    $(window).scrollTop(0);
};

marketstar.Shop.prototype.onIFrameError = function () {
    this.hideLoader();
};
