/**
 * StreamEntry coincides with each <li> within the <ul> that holds a user Activity Stream
 */
_.namespace('marketstar');

/**
 * Constructor
 * 
 * @class StreamEntry
 * @constructor
 * @extends marketstar.DataView
 * @uses Constants
 * @uses AjaxMixins
 * @uses PageMixins
 * @uses UtilsMixins
 */
marketstar.StreamEntry = function(el, data) {
    $.extend(marketstar.StreamEntry.prototype, Constants);
    $.extend(marketstar.StreamEntry.prototype, AjaxMixins);
    $.extend(marketstar.StreamEntry.prototype, PageMixins);
    $.extend(marketstar.StreamEntry.prototype, UtilMixins);
    data.autoRender = true;
    data.renderMethod = 'append';
    this.construct(el, data);
};

marketstar.StreamEntry.prototype = Object.create(marketstar.DataView.prototype); // note custom parent type

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.StreamEntry.prototype.init = function() {
    this.$('a').on('click', this.anchorClickHandler);
    this.localStorage = new marketstar.LocalStorageObserverProxy();
    this.loadIntoModelFromLocalStorage(this.userDataKey);
};

/**
 * Post render() callback.  Contract requirement from DataView.
 * Put anything here that is dependent on the existence of elements in the DOM
 * 
 * @method postRender
 * @async
 */
marketstar.StreamEntry.prototype.postRender = function() {
    //console.log('fetching avatar at ' + this.model.getPath('actor.image.url') + ' for user ' + this.model.getPath('actor.displayName'));
    this.updateImageFromAjax('img.feed-author', this.model.getPath('actor.image.url'));

    if(this.model.getPath('jive.parent'))
    {
        this.basicAuthAjaxCall({
            url:       this.model.getPath('jive.parent.id')
            ,type:      'GET'
            ,basicAuth: this.model.get(this.userDataKey).authHash
            ,modelName: this.model.getPath('jive.parent.id')
            ,success:   $.proxy(this.updateParentCommentName, this)
            ,fail:      this.fail
        });
    }
    if(this.model.getPath('object.id')) {
        // TODO: show the comment string here
        this.basicAuthAjaxCall({
             url:       this.model.getPath('object.id')
            ,type:      'GET'
            ,basicAuth: this.model.get(this.userDataKey).authHash
            ,modelName: this.model.getPath('object.id')
            ,success:   $.proxy(this.updateCommentCount, this)
            ,fail:      this.fail
        });
    }
};

/**
 * Success callback for the ajax call in postRender.
 *
 * @method updateCommentCount
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.StreamEntry.prototype.updateCommentCount = function(requestProps, xhr) {
    var replyCount = this.model.get(requestProps.modelName).replyCount;
    if(replyCount) {
        this.$('.comments').html( replyCount + ' ' + this.simplePluralize(replyCount, 'comment') );
        this.$('.comments').show();
    }
    this.model.setPath(requestProps.modelName, undefined);
}

/**
 * Success callback for the ajax call in postRender.
 *
 * @method updateParentCommentName
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.StreamEntry.prototype.updateParentCommentName = function(requestProps, xhr) {
    var parentName = this.model.get(requestProps.modelName).author.displayName;
    if(parentName) {
        this.$('.parentCommentName').html(parentName);
    }
    this.model.setPath(requestProps.modelName, undefined);
}
