_.namespace('marketstar');

/**
 * Constructor
 * 
 * @class TrainingDetailMain
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.TrainingDetailMain = function(el, data) {
    $.extend(marketstar.TrainingDetailMain.prototype, Constants);
    $.extend(marketstar.TrainingDetailMain.prototype, UtilMixins);
    $.extend(marketstar.TrainingDetailMain.prototype, AjaxMixins);
    $.extend(marketstar.TrainingDetailMain.prototype, PageMixins);
    this.construct(el, data);
};

// NOTE: I'm sick of the constants. So I'm getting rid of them where it's useless, which is pretty much about everywhere. CMS

marketstar.TrainingDetailMain.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.TrainingDetailMain.prototype.init = function() {
    this.localStorage = new marketstar.LocalStorageObserverProxy();

    this.parseQueryParametersIntoModel();
    if( !this.model.getPath('queryParams.trainingId') ) {
        this.goBack();
    }

    this.trainingId = this.model.getPath('queryParams.trainingId');
    this.certRoute = this.model.getPath('queryParams.certification');
    if(this.certRoute != null)
    {
        this.$('a.back-arrow').attr('href', 'certificationDetails.html?certificationId='+this.certRoute);
    }

    //this.localStorage.registerObserver('achievements', this, this.updateModelFromLocalStorage);
    this.localStorage.registerObserver('trainings', this, this.updateModelFromLocalStorage);
    //this.$( 'a.back-arrow' ).on( 'click', this.goBack );

    this.model.set(
        this.templatePathsModelKey
        ,{
            navigation: '_navigation.html'
            , userBar: '_userBar.html'
            , socialLinks: '_socialLinks.html'
            , appVersion: '_appVersion.html'
            , trainingDetailView: '_trainingDetailView.html'
        }
    );
    this.loadIntoModelFromLocalStorage(
         this.userDataKey
        ,$.proxy(this.loadTemplates,   this)
        ,$.proxy(this.redirectToIndex, this)
    );
};

/**
 * onTemplatesReady
 *
 * @method onTemplatesReady
 */
marketstar.TrainingDetailMain.prototype.onTemplatesReady = function() {
    //this.loadIntoModelFromLocalStorage('achievements',$.proxy(this.fetchTraining, this));
    this.loadIntoModelFromLocalStorage('trainings',$.proxy(this.fetchTraining, this), $.proxy(this.getTrainings, this));
};

/**
 * Locate the selected training and add it to the model
 */
marketstar.TrainingDetailMain.prototype.fetchTraining = function() {
    var training
        ,that = this
        ,trainings = this.model.getPath('trainings');//this.model.getPath('achievements');

    training = $.grep(trainings, function(o, i){
        return o && o.Id == that.trainingId;
    })[0];

    this.model.set('training', training);
    this.renderTrainingDetail();
};

/**
 * Use sudo templating to render the training detail.
 *
 * @method templatesReady
 */
marketstar.TrainingDetailMain.prototype.renderTrainingDetail = function() {
    this.renderMyRating();

    var training = this.model.get('training');

    var data = $.extend(
                training
                ,{
                     renderTarget: this.$('div.training-container')
                }
            );
    var template = _.template(this.model.getPath('templates.trainingDetailView'), data);

    var child = new marketstar.TrainingDetailView(
                                        template
                                        ,data
                                    );
    this.addChild(child);
};

/**
 * Render my rating stars
 */
marketstar.TrainingDetailMain.prototype.renderMyRating = function() {
    this.$('.my-rating .rating-stars').addClass(
        this.convertScoreToClassName(
            this.normalizeScore(
                this.model.getPath('training.Rating.MyRating')
            )
        )
    );

    this.$('.hit-area').on('click', $.proxy(this.myRatingOnClick, this));
};

/**
 * Click Handler for My Rating
 *
 * @param e
 */
marketstar.TrainingDetailMain.prototype.myRatingOnClick = function(e) {
    var $x = $(e.currentTarget);
    var classes = ['one', 'two', 'three', 'four', 'five'];

    for(var i=0; i<classes.length; i++) {
        $x.parent().removeClass(classes[i]);
        if($x.hasClass(classes[i])) {
            if( !$x.parent().hasClass(classes[i]) ){
                $x.parent().addClass(classes[i]);
                this.postMyRating(this.wordToDigit(classes[i]));
            }
        }
    }
};

/**
 * POST my rating to MarketStar Api
 *
 * @param rating
 */
marketstar.TrainingDetailMain.prototype.postMyRating = function(rating) {
    var urlParams = {
         urlPrefix:    this.marketstarApiUrlPrefix()
        ,urlPath:      this.marketstarApiAchievementRatingsPath
        ,trainingId: encodeURI(this.model.getPath('training.GamificationId'))
        ,lang:         this.marketstarDefaultLang
    };

    this.marketstarApiAjaxCall({
        url:            this.buildUrl(urlParams)
        , type:         'POST'
        , body:         {rating: rating }
        , modelName:    'achievementPostResult'
        , success:      $.proxy(this.updateRating, this)
        , fail:         $.proxy(this.fail, this)
    });
};

/**
 * Update localstorage w/ new data
 *
 * @param requestProps
 * @param xhr
 */
marketstar.TrainingDetailMain.prototype.updateRating = function(requestProps, xhr) {
    var model = this.model.get('training');
    model.Average = this.model.getPath('achievementPostResult.Average');
    model.MyRating = this.model.getPath('achievementPostResult.MyRating');

    var trainings = this.model.get('trainings');
    //,training = $.grep(trainings, function(i, t) { return i.Id === model.Id; });
    //training = model;
    //var that = this;
    for(var x = 0; x <= trainings.length; x++){
        var training = trainings[x];
        if(training.Id === model.Id){
            training.Average = model.Average;
            training.MyRating = model.MyRating;
            break;
        }
    }

    this.localStorage.save('trainings', trainings, this, this.updateModelFromLocalStorage, true);


    // re-render rating average
    for(var i=0; i<this.children.length; i++){
        var child = this.children[i];
        if (child.toString() === 'TrainingDetailView') {
            child.renderAverageRating();
        }
    }
};


/**
 * Load the trainings. Yep, this is duplicated from the trainingsMain.js file. That's because SolutionSet's code does not provide an easy way to
 * to run methods in a class statically. So, when I attempted to call this method in trainingsMain.js after instantiating it, it caused all sorts
 * of problems with missing contexts and thus, null variable issues. So, I've duplicated it here and will attempt to refactor it when we refactor
 * SolutionSet's code. CMS
 *
 * @method getTrainingsForType
 * @async
 * @param typeId {Number}
 */
marketstar.TrainingDetailMain.prototype.getTrainings = function() {
    this.trainingsLoadAttempts++;
    var urlParams = {
        urlPrefix:  this.marketstarApiUrlPrefix()
        ,urlPath:   'trainings?lang={lang}'
        ,lang:      this.marketstarDefaultLang
    };

    this.marketstarApiAjaxCall({
        url:       this.buildUrl(urlParams)
        ,type:      'GET'
        ,modelName: 'trainings'
        ,success:   $.proxy(this.getTrainingsSuccess, this)
        ,fail:      $.proxy(this.getTrainingsFail, this)
    });
};

marketstar.TrainingDetailMain.prototype.getTrainingsSuccess = function(requestProps, xhr) {
    if(xhr) {
        this.localStorage.save('trainings', this.model.getPath('trainings'), this, null, null);
        this.loadIntoModelFromLocalStorage('trainings',$.proxy(this.fetchTraining, this));
    }
};
marketstar.TrainingDetailMain.prototype.getTrainingsFail = function(requestProps, xhr) {
    // TODO: Fail
    console.log(xhr);
};