_.namespace('marketstar');

/**
 * Constructor
 * 
 * @class TrainingDetailView
 * @constructor
 * @extends marketstar.DataView
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.TrainingDetailView = function(el, data) {
    $.extend(marketstar.TrainingDetailView.prototype, Constants);
    $.extend(marketstar.TrainingDetailView.prototype, AjaxMixins);
    $.extend(marketstar.TrainingDetailView.prototype, PageMixins);
    $.extend(marketstar.TrainingDetailView.prototype, UtilMixins);
    this.construct(el, data);  
};

// subclass our _.DataView subclass
marketstar.TrainingDetailView.prototype = Object.create(marketstar.DataView.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.TrainingDetailView.prototype.init = function() {
    this.localStorage = new marketstar.LocalStorageObserverProxy();
    this.model.set(
        this.templatePathsModelKey
        ,{
            trainingModule: '_trainingModule.html'
            , trainingScorm: '_trainingScorm.html'
        }
    );
    this.loadIntoModelFromLocalStorage(
        this.userDataKey
    );
};

/**
 * Render the page
 *
 * @method templatesReady
 */
marketstar.TrainingDetailView.prototype.templatesReady = function() {
    var  module
        ,modules = this.model.getPath('ConnectTrainingModules') || this.model.getPath('ScormTrainingContent')
        ,$renderTarget = $('ul.training-modules');

    $renderTarget.empty();


    if(Array.isArray(modules)) {
        for(var i=0; i<modules.length; i++) {
            module = modules[i];
            module.ConnectUrl = this.buildMarketStarUrl("trainingcontent/get", { trainingId : module.TrainingId, contentId : module.Id } );

            template = _.template(this.model.getPath('templates.trainingModule'), module);
            data = {
                renderTarget: $renderTarget
            };
            child = new marketstar.TrainingModule(
                                                template
                                                ,$.extend(data, module)
                                            );
            this.addChild(child);
        }
    }
    else {
        // Scorm Trainings.
        module = modules;
        module.ConnectUrl = this.buildMarketStarUrl("trainingcontent/get", { trainingId : module.TrainingId, contentId : module.Id } );

        template = _.template(this.model.getPath('templates.trainingScorm'), module);
        data = {
            renderTarget: $renderTarget
        };
        child = new marketstar.TrainingModule(
            template
            ,$.extend(data, module)
        );
        this.addChild(child);
    }

};

/**
 * Post-render callback for marketstar.DataView.
 *
 * @method postRender
 */
marketstar.TrainingDetailView.prototype.postRender = function() {
    this.loadTemplates();
    this.renderAverageRating();
};

/**
 * Render the average rating
 */
marketstar.TrainingDetailView.prototype.renderAverageRating = function() {
    this.$('.avg-rating').removeClass().addClass('avg-rating rating-average');
    this.$('.avg-rating').addClass(
        this.convertScoreToClassName(
            this.normalizeScore(
                this.model.getPath('Average') || this.model.getPath('Rating.Average')
            )
        )
    );
};

marketstar.TrainingDetailView.prototype.toString = function() {
    return "TrainingDetailView";
};


/**
 * Constructor
 *
 * An "inner" class for the training module <li>'s.
 * 
 * @class TrainingModule
 * @constructor
 * @extends marketstar.DataView
 * @uses PageMixins
 * @uses UtilMixins
 */
marketstar.TrainingModule = function(el, data) {
    $.extend(marketstar.TrainingModule.prototype, PageMixins);
    $.extend(marketstar.TrainingModule.prototype, UtilMixins);
    this.construct(el, data);
};

marketstar.TrainingModule.prototype = Object.create(marketstar.DataView.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.TrainingModule.prototype.init = function() {

};

/**
 * sudo post render
 */
marketstar.TrainingModule.prototype.postRender = function() {
    //this.$('.training-link h5').on('click', this.showTrainingButtons);
    //this.$('.training-link').on('click', this.showTrainingButtons);
    this.$('a.take-quiz').on('click', $.proxy(this.takeQuizClick, this));
    
    this.$('div.view-training').on('touchend', $.proxy(function(e) {
        if (Config.openTrainingsUsingInAppBrowser) {
            this.openAnchorInAppBrowser(e);
        }
        else {
            this.openAnchorInSystemBrowser(e);
        }
    }, this));
    
    var completed = this.model.get('UserModuleCompleted')
        ,onFirstAttempt = this.model.getPath('UserModuleCompleted.OnFirstAttempt');

    if (completed) {
        this.$('.check-mark').show();
        if (onFirstAttempt) {
            this.$('.round-mark').show();
        }
		this.$('.complete-percent').show();
		this.$('a.take-quiz').html('View Results');
    }
    else {
        this.$('.check-mark').hide();
        this.$('.round-mark').hide();
		this.$('.complete-percent').hide();
    }
};

marketstar.TrainingModule.delayClick = false;
marketstar.TrainingModule.prototype.showTrainingButtons = function(e) {
    if(!this.delayClick) {
        this.delayClick = true; // Wait until this process is complete before continuing.
        e.stopPropagation();

        var $modActions = $(this).parent().find('.module-actions');
        if($(e.currentTarget).is('h5')) {
            $modActions = $(this).parent().parent().find('.module-actions');
        }
        if ($modActions.is(':visible')) {
            $modActions.hide();
        } else {
            $modActions.show();
        }
        var that = this;
        setTimeout(function() {
            that.delayClick = false;
        }, 300); // Prevents open/close bug on some devices.
    }
}

marketstar.TrainingModule.prototype.takeQuizClick = function() {
    var trainingModuleId = this.model.get('Id');
    var trainingId = this.model.get('TrainingId');
    var returnUrl = 'trainingDetail.html?trainingId=' + trainingId;
    window.location.href = 'trainingQuiz.html?trainingModuleId=' + trainingModuleId + '&trainingId=' + trainingId + '&returnUrl=' + encodeURIComponent(returnUrl);
    return false;
};
