_.namespace('marketstar');

/**
 * Constructor
 * 
 * @class TrainingQuizMain
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.TrainingQuizMain = function(el, data) {
    $.extend(marketstar.TrainingQuizMain.prototype, Constants);
    $.extend(marketstar.TrainingQuizMain.prototype, UtilMixins);
    $.extend(marketstar.TrainingQuizMain.prototype, AjaxMixins);
    $.extend(marketstar.TrainingQuizMain.prototype, PageMixins);
    this.construct(el, data);
};

marketstar.TrainingQuizMain.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.TrainingQuizMain.prototype.init = function() {
    this.localStorage = new marketstar.LocalStorageObserverProxy();

    this.parseQueryParametersIntoModel();
    this.trainingModuleId = this.model.getPath('queryParams.trainingModuleId')
    this.trainingId = this.model.getPath('queryParams.trainingId');

    this.model.set(
         this.templatePathsModelKey
        ,{
            navigation: '_navigation.html'
            , userBar: '_userBar.html'
            , socialLinks: '_socialLinks.html'
            , appVersion: '_appVersion.html'
        }
    );
    this.loadIntoModelFromLocalStorage(
         this.userDataKey
        ,$.proxy(this.loadTemplates,   this)
        ,$.proxy(this.redirectToIndex, this)
    );    
};

/**
 * onTemplatesReady callback
 *
 * @method onTemplatesReady
 */
marketstar.TrainingQuizMain.prototype.onTemplatesReady  = function() {
    this.showLoader();
    var that = this;

    this.$('a.back-arrow').on('click', function(){
        window.location = "trainingDetail.html?trainingId="+that.trainingId;
    });
    var userId = this.model.getPath(this.userDataKey +'.id');
    this.frame = this.$('iframe')
        ,timestamp = Math.floor( new Date().getTime() / 1000 )
        ,urlParams = {
            urlPrefix: Config.marketstarUrl
            ,urlPath:   'Quiz?UserId={userId}&ApiKey={apiKey}&Timestamp={timestamp}&Signature={signature}&trainingModuleId={trainingModuleId}'
            ,userId:  userId
            ,apiKey: Config.marketstarApiKey
            ,timestamp: timestamp
            ,signature: this.generateMarketstarApiSignature(timestamp, userId)
            ,trainingModuleId: this.trainingModuleId
        };
    this.frame.attr('src', this.buildUrl(urlParams));
    this.frame.on('load', $.proxy(this.onResize, this));
    this.frame.on('error', $.proxy(this.onIFrameError, this));
};

marketstar.TrainingQuizMain.prototype.bindScroller = function() {
    // don't use iScroll
};

/**
 * onResize callback
 */
marketstar.TrainingQuizMain.prototype.onResize = function() {
    this.hideLoader();
    this.frame.height(this.frame.contents().height());
    $(window).scrollTop(0);
};

/**
 * onIFrameError callback
 */
marketstar.TrainingQuizMain.prototype.onIFrameError = function() {
    this.hideLoader();
};
