_.namespace('marketstar');

/**
 * Constructor
 *
 * @class TrainingsMain
 * @constructor
 * @extends _.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses UtilsMixins
 * @uses PageMixins
 */
marketstar.TrainingsMain = function (el, data) {
  $.extend(marketstar.TrainingsMain.prototype, Constants);
  $.extend(marketstar.TrainingsMain.prototype, UtilMixins);
  $.extend(marketstar.TrainingsMain.prototype, AjaxMixins);
  $.extend(marketstar.TrainingsMain.prototype, PageMixins);
  this.construct(el, data);
};

marketstar.TrainingsMain.prototype = Object.create(marketstar.ViewController.prototype);

/**
 * Sudo init()
 *
 * @method init
 */
marketstar.TrainingsMain.prototype.init = function () {
  this.localStorage = new marketstar.LocalStorageObserverProxy();
  this.refreshRequest = false;

  this.model.set(
    'templatePaths'
    , {
      navigation: '_navigation.html'
      , userBar: '_userBar.html'
      , socialLinks: '_socialLinks.html'
      , appVersion: '_appVersion.html'
      , training: '_training.html'
    }
  );
  this.model.set('trainingTypeId', -1);

  this.loadIntoModelFromLocalStorage(
    this.userDataKey
    , $.proxy(this.loadTemplates, this)
    , $.proxy(this.redirectToIndex, this)
  );
};

/**
 * Render the page
 *
 * @method templatesReady
 */
marketstar.TrainingsMain.prototype.onTemplatesReady = function () {
  //this.getTrainings();
  //this.getTrainingTypes();
};

/**
 * Load the training types for ddl
 *
 * @method getTrainingTypes
 * @async
 */
marketstar.TrainingsMain.prototype.getTrainingTypes = function () {

  this.loadIntoModelFromLocalStorage('trainingTypes', $.proxy(this.fetchTraining, this));

  if (!this.refreshRequest && this.model.get('trainingTypes')) {
    this.getTrainingTypesSuccess(null, null);
    return;
  } else {

    this.refreshRequest = false;

    var urlParams = {
      urlPrefix: this.marketstarApiUrlPrefix()
      , urlPath: this.marketstarApiTrainingTypesPath
      , lang: this.marketstarDefaultLang
    };

    this.marketstarApiAjaxCall({
      url: this.buildUrl(urlParams)
      , type: 'GET'
      , modelName: 'trainingTypes'
      , success: $.proxy(this.getTrainingTypesSuccess, this)
      , fail: $.proxy(this.getTrainingTypesFail, this)
    });
  }
};

/**
 * Adds the training types as <option>'s to the <select>.
 * Success callback for getTrainingTypes
 *
 * @method getTrainingTypesSucess
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.TrainingsMain.prototype.getTrainingTypesSuccess = function (requestProps, xhr) {
  var modelName = 'trainingTypes'
    , defaultElement = {
      Id: -2
      , Name: this.allTrainingsText
    }
    , props = {
      selector: '.selector-container .selector'
      , coll: this.model.get(modelName)
      , idField: 'Id'
      , nameField: 'Name'
      , onChangeHandler: $.proxy(this.getTrainingsForTypeHandler, this)
    };

  if (xhr) {
    this.model.get(modelName).push(defaultElement);
    this.localStorage.save(modelName, this.model.get(modelName).reverse(), this, this.updateModelFromLocalStorage, null);
  }

  this.updateUl(props);
  //var optVal = parseInt(this.$('.selector-container .selector option:last-child').val());

  //optVal += 1;
  this.$('.selector-container .selector').append("<option value='-1' selected='selected'>Incomplete</option>");
  //optVal += 1;
  this.$('.selector-container .selector').append("<option value='0'>Complete</option>");
};

/**
 * Failure callback for getTrainingTypes
 *
 * @method getUserStreamsFail
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.TrainingsMain.prototype.getTrainingTypesFail = function (requestProps, xhr) {
  // TODO: Fail
};

/**
 * Event handler for the <select>.onChange()
 *
 * @method getTrainingsForTypeHandler
 * @event change
 * @param e {Object}
 */
marketstar.TrainingsMain.prototype.getTrainingsForTypeHandler = function (e) {
  this.model.set('trainingTypeId', parseInt(e.target.options[e.target.options.selectedIndex].value));
  this.getTrainingsSuccess(null, null);
};

/**
 * Load the trainings for type.
 *
 * @method getTrainingsForType
 * @async
 * @param typeId {Number}
 */
marketstar.TrainingsMain.prototype.getTrainings = function () {

  this.loadIntoModelFromLocalStorage('trainings', $.proxy(this.fetchTraining, this));

  if (!this.refreshRequest && this.model.getPath('trainings') !== undefined && this.model.getPath('trainings').length > 0) {
    this.getTrainingsSuccess();
  }
  else {
    this.refreshRequest = false;

    var urlParams = {
      urlPrefix: this.marketstarApiUrlPrefix()
      , urlPath: 'trainings?mobileOnly=true'
      , lang: this.marketstarDefaultLang
    };

    this.marketstarApiAjaxCall({
      url: this.buildUrl(urlParams)
      , type: 'GET'
      , modelName: 'trainings'
      , success: $.proxy(this.getTrainingsSuccess, this)
      , fail: $.proxy(this.getTrainingsFail, this)
    });
  }
};


/**
 * Render <li>'s unto the <ul> containing stream activities using sudo template 'templates.training'
 * Success callback for getTrainings.
 *
 * @method getTrainingsSuccess
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.TrainingsMain.prototype.getTrainingsSuccess = function (requestProps, xhr) {
  var $container = this.$('ul.training-list')
    , modelName = 'trainings'
    , i
    , training
    , trainings = this.model.getPath('trainings')
    , trainingTypeId = this.model.get('trainingTypeId');


  $container.empty();

  if (xhr) {
    this.localStorage.save(modelName, trainings, this, this.updateModelFromLocalStorage, null);
  }

  // TODO: sort trainings by date

  for (i = 0; i < trainings.length; i++) {
    training = trainings[i];

    if ((trainingTypeId == -2) ||
      (trainingTypeId == -1 && training.PercentComplete < 100 && training.UserTrainingCompleted == null) ||
      (trainingTypeId == 0 && training.UserTrainingCompleted != null) ||
      (trainingTypeId == training.TrainingTypeId)) {

      var template = _.template(this.model.getPath('templates.training'), training);
      var data = {
        renderTarget: $container
      };
      var child = new marketstar.Training(template, $.extend(data, training));
      this.addChild(child);
    }
  }
};

/**
 * Fail callback for getTrainings()
 *
 * @method getTrainingsFail
 * @param requestProps {Object}
 * @param xhr          {Object}
 */
marketstar.TrainingsMain.prototype.getTrainingsFail = function (requestProps, xhr) {
  // TODO: Fail
};

/*
marketstar.TrainingsMain.prototype.bindScroller = function () {
  var pullDownEl = document.getElementById('pullDown')
    , pullDownOffset = pullDownEl.offsetHeight
    , that = this;

  this.iScroll = new iScroll('iscroller', {
    useTransition: false,
    topOffset: pullDownOffset,
    checkDOMChanges: true,
    hScrollbar: false,
    vScrollbar: false,

    onRefresh: function () {
      if (pullDownEl.className.match('loading')) {
        pullDownEl.className = '';
        pullDownEl.querySelector('.pullDownLabel').innerHTML = 'Pull down to refresh...';
      }
    },
    onBeforeScrollStart: function (e) {
      var nodeType = e.explicitOriginalTarget ? e.explicitOriginalTarget.nodeName.toLowerCase() : (e.target ? e.target.nodeName.toLowerCase() : '');

      if (nodeType != 'select' && nodeType != 'option' && nodeType != 'input' && nodeType != 'textarea') {
        e.preventDefault();
      }
      that.$('#pullDown').css('visibility', 'visible');

    },
    onScrollMove: function () {
      if (this.y > 5 && !pullDownEl.className.match('flip')) {
        pullDownEl.className = 'flip';
        pullDownEl.querySelector('.pullDownLabel').innerHTML = 'Release to refresh...';
        this.minScrollY = 0;
      } else if (this.y < 5 && pullDownEl.className.match('flip')) {
        pullDownEl.className = '';
        pullDownEl.querySelector('.pullDownLabel').innerHTML = 'Pull down to refresh...';
        this.minScrollY = -pullDownOffset;
      }
    },
    onScrollEnd: function () {
      if (pullDownEl.className.match('flip')) {
        pullDownEl.className = 'loading';
        pullDownEl.querySelector('.pullDownLabel').innerHTML = 'Loading...';
        that.onPullToRefresh();
      }

      that.$('#pullDown').css('visibility', 'hidden');

    }
  });


  this.iScroll = new iScroll('navscroller', {
    useTransition: false,
    checkDOMChanges: true,


    onBeforeScrollStart: function (e) {
      //e.stopPropagation();

      this.maxScrollY = (parseInt($(window).height()) - parseInt($('.nav-container').height() + 10));

      var nodeType = e.explicitOriginalTarget ? e.explicitOriginalTarget.nodeName.toLowerCase() : (e.target ? e.target.nodeName.toLowerCase() : '');

      if (nodeType != 'select' && nodeType != 'option' && nodeType != 'input' && nodeType != 'textarea') {
        e.preventDefault();
      }
    }

  });


};
    */
/**
 * On Pull To Refresh Action
 */
/*
marketstar.TrainingsMain.prototype.onPullToRefresh = function () {

  this.refreshRequest = true;
  this.getTrainingTypes();
  this.refreshRequest = true;
  $('.training-list').empty();
  this.getTrainings();
  //this.getStreamActivities(parseInt($('.selector option:selected').val()), null);
};*/