﻿//Generates a survey in an empty html parent element
function generateSurveyUI(elementId, survey) {
    if (survey.SurveyType == 'prechatsurvey' && (survey.Header == null || survey.Header == '')) {
        jQuery('#prechathead').show();
    }
    else {
        jQuery('#prechathead').hide();
    }

    var parent = document.getElementById(elementId);
    if (parent != null) {
        //Verify it is empty
        parent.innerHTML = '';

        //Add the header
        if (survey.Header != null && survey.Header != '') {
            var header = document.createElement('div');
            header.id = 'surveyHeader';
            header.insertAdjacentHTML('beforeend', survey.Header);
            parent.appendChild(header);
        }

        //Add the required field message
        var requiredMsg = document.createElement('div');
        requiredMsg.id = 'requiredFlag';
        requiredMsg.style.display = 'none';
        parent.appendChild(requiredMsg);

        //Add the questions
        var questions = document.createElement('div');
        questions.id = 'surveyQuestions';
        parent.appendChild(questions);
        if (survey.Questions != null) {
            for (var i = 0; i < survey.Questions.Questions.length; i++) {
                var question = survey.Questions.Questions[i];
                generateQuestion(question, survey, questions);
            }
        }

        //Default survey
        if (survey.SurveyID == -1 && survey.SurveyType == 'prechatsurvey') {
           
            var defaultSurvey = '<input type="text" class="input-block-level" name="name" id="name" data-required="Please enter your name" placeholder="*Your Name" data-istype="IsUserName" />';
            defaultSurvey += '<input type="email" class="input-block-level" name="email" id="email" placeholder="Email Address" data-istype="IsEmail" />';
            jQuery(parent).children()[1].insertAdjacentHTML('beforeend', defaultSurvey);
        }

        //Submit button
        if (survey.SurveyType == 'prechatsurvey') {
            parent.insertAdjacentHTML('beforeend', '<a href="#" class="btn btn-block btn-large" id="start" onclick="sendSurveyResults(\'prechat\', \'' + elementId + '\');">' + survey.ButtonText + '</a>');
        }
        else if (survey.SurveyType == 'postchatsurvey') {
            parent.insertAdjacentHTML('beforeend', '<a href="#" class="btn btn-block btn-large" id="postchatsubmit" onclick="sendSurveyResults(\'postchat\', \'' + elementId + '\');">' + survey.ButtonText + '</a>');
        }
        else if (survey.SurveyType == 'notheresurvey') {
            parent.insertAdjacentHTML('beforeend', '<a href="#" class="btn btn-block btn-large" id="notonlinesubmit" onclick="sendSurveyResults(\'notonline\', \'' + elementId + '\');">' + survey.ButtonText + '</a>');
        }

        //Survey ID
        parent.insertAdjacentHTML('beforeend', '<input type="hidden" id="surveyID" value="' + survey.SurveyID + '" />');

        //Thanks
        if (survey.ShowThanks != null && survey.ShowThanks != '') {
            if (survey.ThanksText == null)
                survey.ThanksText = ' ';
            parent.insertAdjacentHTML('beforeend', '<div id="surveyThanks" style="display: none;">' + survey.ThanksText + '</div>');
        }
    }
}

function generateQuestion(question, survey, parent) {
    switch (question.Type) {
        case -8174:
            /*unknown*/;
        case -3:
            /*Column Break*/;
        case -2:
            /*Page Break*/;
        case -1:
            hiddenFieldGeneration(question, parent);
            break;
        case 0:
            plainTextGeneration(question, parent);
            break;
        case 1:
            textBoxGeneration(question, parent);
            break;
        case 2:
            multiLineTextBoxGeneration(question, parent);
            break;
        case 3:
            comboBoxGeneration(question, survey.Questions.Choices, parent);
            break;
        case 4:
            radioButtonGeneration(question, survey.Questions.Choices, parent);
            break;
        case 5:
            checkBoxGeneration(question, survey.Questions.Choices, parent);
            break;
        case 6:
            radioButtonListGeneration(question, survey.Questions.Choices, parent);
            break;
        case 7:
            checkBoxListGeneration(question, survey.Questions.Choices, parent);
            break;
        case 8:
            visitorLanguageGeneration(question, survey.Questions.Choices, parent);
            break;
        case 9:
            NPSGeneration(question, parent);
            break;
        default:
            /*Survey Retrieval Failed*/
            break;
    }
}

function getType(question) {
    if (question.IsEmail == true) {
        return "IsEmail";
    }
    else if (question.IsPhone == true) {
        return "IsPhone";
    }
    else if (question.IsUserName == true) {
        return "IsUserName";
    }
    else if (question.IsKBQuestion == true) {
        return "IsKBQuestion";
    }
    return '';
}

function hiddenFieldGeneration(question, parent) {
    parent.insertAdjacentHTML('beforeend', '<input type="hidden" name="' + question.QuestionID + '" data-questionid="' + question.QuestionID + '" value="' + question.QuestionText + '"  />');
}

function plainTextGeneration(question, parent) {
    parent.insertAdjacentHTML('beforeend', '<div class="plaintext">' + question.PreFormat + question.QuestionText + question.PostFormat + '</div>');
}

function textBoxGeneration(question, parent) {
    var type = getType(question);
    var required = '';
    var rText = '';
    if (question.Required != null) {
        required = question.Required;
        rText = '*';
    }
    var insertHTML = '<input type="text" class="input-block-level bootInput" data-required="' + required.toString() + '" id="' + type + '"';
    insertHTML += 'data-questionid="' + question.QuestionID + '" placeholder=" ' + rText + question.QuestionText + '" data-istype="' + type + '" value="" />';

    parent.insertAdjacentHTML('beforeend', insertHTML);
}

function getRequiredBool(question) {
    if (question.Required != null)
        return question.Required;
    else
        return '';
}

function getrText(question) {
    if (question.Required != null)
        return '*';
    else
        return '';
}

function multiLineTextBoxGeneration(question, parent) {
    var type = getType(question);
    var rText = getrText(question);
    var required = getRequiredBool(question);

    var insertHTML = '<textarea  class="input-block-level bootInput textareaBoot" rows="4" cols="50" data-required="' + required.toString() + '" id=""';
    insertHTML += 'data-questionid="' + question.QuestionID + '" placeholder="' + rText + question.QuestionText + '" data-istype="' + type + '" value="" />'
    parent.insertAdjacentHTML('beforeend', insertHTML);
    }

function comboBoxGeneration(question, choices, parent) {
    var type = getType(question);
    var rText = getrText(question);
    var required = getRequiredBool(question);

    var htmlString = '<div>' + question.PreFormat + rText + question.QuestionText;
    htmlString += question.PostFormat + '</div>';
    htmlString += '<select class="input-block-level bootInput" data-questionid="' + question.QuestionID + '" data-istype="' + type + '">';
    for (var i = 0; i < choices.length; i++) {
        if (question.QuestionID == choices[i].QuestionID) {
            htmlString += '<option value="' + choices[i].Choice + '"';

            if (choices[i].Choice.toLowerCase() == "english")
                htmlString += 'selected="selected"';

            htmlString += '>' + choices[i].Choice + '</option>';
        }
    }

    htmlString += '</select>';
    parent.insertAdjacentHTML('beforeend', htmlString);
}

function radioButtonGeneration(question, choices, parent) {
    var type = getType(question);
    var rText = getrText(question);
    var required = getRequiredBool(question);

    var htmlString = '<div id="" style="padding-top:10px; padding-bottom:10px;" data-required="' + required.toString() + '"><div class="checkbox-list-question radiobutton">' + question.PreFormat + rText + question.QuestionText;
    htmlString += question.PostFormat + '</div><div class="checkbox-list-choices">';
    for (var i = 0; i < choices.length; i++) {
        if (question.QuestionID == choices[i].QuestionID) {
            htmlString += '<div style=""><input type="radio" name="' + question.QuestionID + '" id="' + question.QuestionID +'" data-istype="' + type + '" data-questionid="' + question.QuestionID + '" class="checkbox-choice bootInput bootRadio" value="' + choices[i].Choice + '"></input><span style="margin-left: 10px;">' + choices[i].Choice + '</span></div>';
        }
    }
    htmlString += '</div>';
    parent.insertAdjacentHTML('beforeend', htmlString);
}

function checkBoxGeneration(question, choices, parent) {
    var type = getType(question);
    var rText = getrText(question);
    var required = getRequiredBool(question);

    var htmlString = '<div id="" style="padding-top:10px; padding-bottom:10px;" data-required="' + required.toString() + '"><div class="checkbox-list-question checkboxOptions">' + question.PreFormat + rText + question.QuestionText;
    htmlString += question.PostFormat + '</div><div class="checkbox-list-choices">';
    for (var i = 0; i < choices.length; i++) {
        if (question.QuestionID == choices[i].QuestionID) {
            htmlString += '<div><input type="checkbox" name="' + question.QuestionID + '" class="checkbox-choice bootInput bootCheckbox" data-istype="' + type + '" value="' + choices[i].Choice + '"></input><span style="margin-left: 10px;">' + choices[i].Choice + '</span></div>';
        }
    }
    htmlString += '</div>';
    parent.insertAdjacentHTML('beforeend', htmlString);
}

function radioButtonListGeneration(question, choices, parent) {
    var rText = getrText(question);
    var required = getRequiredBool(question);

    var htmlString = '<div id = "" style="padding-top:10px; padding-bottom:10px;" data-required="' + required.toString() + '"><div class="checkbox-list-question bootInput radiobutton">' + question.PreFormat + rText + question.QuestionText + question.PostFormat + '</div><div class="checkbox-list-choices radiobutton">';
    for (var i = 0; i < choices.length; i++) {
        if (question.QuestionID == choices[i].QuestionID) {
            htmlString += '<div style=""><input type="radio" name="' + question.QuestionID + '" class="checkbox-choice bootInput bootRadio" value="' + choices[i].Choice + '"></input><span style="margin-left: 10px;">' + choices[i].Choice + '</span></div>';
        }
    }
    htmlString += '</div>';
    parent.insertAdjacentHTML('beforeend', htmlString);
}

function checkBoxListGeneration(question, choices, parent) {
    var rText = getrText(question);
    var required = getRequiredBool(question);

    var htmlString = '<div style="padding-top:10px; padding-bottom:10px;" data-required="' + required.toString() + '"><div class="checkbox-list-question bootInput checkboxOptions" >' + question.PreFormat + rText + question.QuestionText + question.PostFormat + '</div><div class="checkbox-list-choices checkbox">';
    for (var i = 0; i < choices.length; i++) {
        if (question.QuestionID == choices[i].QuestionID) {
            htmlString += '<div style=""><input type="checkbox" class="checkbox-choice bootInput bootCheckbox checkbox" value="' + choices[i].Choice + '"></input><span style="margin-left: 10px;">' + choices[i].Choice + '</span></div>';

        }
    }
    htmlString += '</div>';
    parent.insertAdjacentHTML('beforeend', htmlString);
}

function visitorLanguageGeneration(question, choices, parent) {
    var rText = getrText(question);
    var required = getRequiredBool(question);

    var htmlString = '<p>' + rText + question.QuestionText + '</p><select class="input-block-level bootInput" name="visitor language" data-required="' + required.toString() + '" data-questionid="' + question.QuestionID + '">';
    for (var i = 0; i < choices.length; i++) {
        if (question.QuestionID == choices[i].QuestionID) {
            htmlString += '<option value="' + choices[i].AliasName + '"';

            if (choices[i].Choice.toLowerCase() == "english")
                htmlString += 'selected="selected"';

            htmlString += '>' + choices[i].Choice + '</option>';
        }
    }
    htmlString += '</select>';
    parent.insertAdjacentHTML('beforeend', htmlString);
}

function NPSGeneration(question, parent) {    
    var NPSString = '<table class="npsStyling" width="100%" data-questiontype="NPS" border="0"><tbody><tr><td colspan="2" valign="top">';
    NPSString += '<table class="npsStyling" border="0" cellspacing="0"><tbody><tr><td colspan="10"><b>How likely is it that you would recommend our company to a friend or colleague?</b></td></tr><tr>';
    NPSString += '<td colspan="5">Not at all Likely</td><td align="right" colspan="5">Very Likely</td></tr>';
    NPSString += '<tr><td align="center"><input type="radio" name="' + question.QuestionID + '" value="0"></td>'
    NPSString += '<td align="center"><input type="radio" name="' + question.QuestionID + '" value="1"></td>';
    NPSString += '<td align="center"><input type="radio" name="' + question.QuestionID + '" value="2"></td>';
    NPSString += '<td align="center"><input type="radio" name="' + question.QuestionID + '" value="3"></td>';
    NPSString += '<td align="center"><input type="radio" name="' + question.QuestionID + '" value="4"></td>';
    NPSString += '<td align="center"><input type="radio" name="' + question.QuestionID + '" value="5"></td>';
    NPSString += '<td align="center"><input type="radio" name="' + question.QuestionID + '" value="6"></td>';
    NPSString += '<td align="center"><input type="radio" name="' + question.QuestionID + '" value="7"></td>';
    NPSString += '<td align="center"><input type="radio" name="' + question.QuestionID + '" value="8"></td>';
    NPSString += '<td align="center"><input type="radio" name="' + question.QuestionID + '" value="9"></td>';
    NPSString += '<td align="center"><input type="radio" name="' + question.QuestionID + '" value="10"></td>';
    NPSString += '</tr>';
    NPSString += '<tr><td align="center">0</td><td align="center">1</td><td align="center">2</td><td align="center">3</td><td align="center">4</td><td align="center">5</td><td align="center">6</td><td align="center">7</td><td align="center">8</td><td align="center">9</td><td align="center">10</td></tr>';
    NPSString += '</tbody></table>';
    NPSString += '</td></tr>';
    NPSString += '</tbody></table>';
    parent.insertAdjacentHTML('beforeend', NPSString);
}

function sendSurveyResults(type, elementId) {
     var parent = document.getElementById(elementId);
     if (parent != null) {
         var answers = new Array();
         var name = "Guest";
         var email = "";
         var customProps = {};
         var required = '';

         var surveyResults = new Object();

         surveyResults.SurveyId = 0;
         surveyResults.ChatId = '';
         surveyResults.SurveyType = type;
         surveyResults.SocialLoginId = '';

         if (type == 'postchat') {
             surveyResults.ChatId = chatid;
         }
         var questions = document.getElementById('surveyQuestions')
         for (var i = 0; i < questions.childNodes.length; i++) {
             var question = questions.childNodes[i];
             if (question.nodeType == 1) {
                 var answer = new Object();
                 answer.QuestionId = 0;
                 answer.SubQuestionId = 0;
                 answer.Text = "";
                 answer.IsUsername = false;
                 answer.IsEmail = false;
                 answer.IsPhone = false;
                 answer.IsDeptName = false;
                 answer.RequiresAlert = false;

                 if (jQuery(question).attr('data-questiontype') == "NPS") {
                     var allRadioButtons = jQuery(jQuery(parent).children()[i]).find('input');
                     answer.QuestionId = jQuery(jQuery(allRadioButtons)[i]).attr('name');
                     for (var x = 0; x < allRadioButtons.length; x++) {
                         var attr = jQuery(jQuery(allRadioButtons)[x]).attr('checked');
                         if (typeof attr !== 'undefined' && attr !== false) {
                             answer.Text = jQuery(jQuery(allRadioButtons)[x]).attr('value');
                             break;
                         }
                     }

                 }
                 else if (jQuery(jQuery(question).children()[0]).hasClass('radiobutton') || jQuery(jQuery(question).children()[0]).hasClass('checkboxOptions')) {

                     var selectedinput = jQuery(jQuery(question).children()[1]).find('input');
                     for (var x = 0; x < selectedinput.length; x++) {
                         answer.QuestionId = jQuery(jQuery(selectedinput)[x]).attr('data-questionid');

                         if (jQuery(jQuery(question).children()[0]).hasClass('radiobutton')) {
                             if (jQuery(jQuery(selectedinput)[x]).is(':checked')) {
                                 answer.Text = jQuery(jQuery(selectedinput)[x]).val();
                                 break;
                             }
                         }
                         else {
                             if (answer.Text == "") {
                                 if (jQuery(jQuery(selectedinput)[x]).is(':checked')) {
                                     answer.Text = jQuery(jQuery(selectedinput)[x]).val();
                                 }
                             }
                             else {
                                 answer.Text += ',' + jQuery(jQuery(selectedinput)[x]).val();
                             }
                         }
                     }

                     if (answer.Text == "" && jQuery(question).attr('data-required') != "") {
                         required = jQuery(question).attr('data-required');
                         break;
                     }
                 }
                else {
                    if (jQuery(question).attr('data-required') != "" && jQuery(question)[0].value == '') {
                        required = jQuery(question).attr('data-required');
                        break;
                    }

                     answer.QuestionId = jQuery(question).attr('data-questionid');
                     answer.Text = jQuery(question).val();
                     var postType = jQuery(question).attr('data-istype');

                     if (postType == "IsEmail") {
                         answer.IsEmail = true;
                         if (jQuery(question).attr('value').length > 0)
                             email = jQuery(question).attr('value');
                     }
                     else if (postType == "IsPhone") {
                         answer.IsPhone = true;
                     }
                     else if (postType == "IsUserName") {
                         answer.IsUsername = true;
                         if (jQuery(question).attr('value').length > 0)
                             name = jQuery(question).attr('value');
                     }
                     else if (postType == "IsKBQuestion") {
                         answer.IsKBQuestion = true;
                     }
                 }
                 if (typeof (answer.QuestionId) != 'undefined' || document.getElementById('surveyID').value < 0) {
                     answers.push(answer);
                 }
             }
        } 
         if (required != '') {
             document.getElementById('requiredFlag').innerHTML = required;
             document.getElementById('requiredFlag').style.display = 'block';
         }
         else if (document.getElementById('surveyID').value < 0) {
             surveyResults.Answers = answers;
             surveyResults.SurveyId = document.getElementById('surveyID').value;
             receiveDefaultSurveyResults(surveyResults);
         }
         else {
             surveyResults.Answers = answers;
             surveyResults.SurveyId = document.getElementById('surveyID').value;
             receiveSurveyResults(surveyResults);
         }
     }
     if ((type == "postchat" || type == "notonline") && required == '') {
         jQuery('#survey').show();
         jQuery('#survey').children().hide();
         jQuery('#builtSurvey').show();
         jQuery('#builtSurvey').children().hide();
         jQuery('#surveyThanks').show();
     }
}