sudo.namespace('marketstar');

/*
 prevent touch move for iScroller execution issues
 */
 if (typeof iScroll !== "undefined") {
    document.addEventListener('touchmove', function (e) { e.preventDefault(); }, false);
 }

/**
 * Constructor
 *
 * @class ViewController
 * @constructor
 * @extends sudo.ViewController
 * @uses Constants
 * @uses AjaxMixins
 * @uses PageMixins
 */
marketstar.ViewController = function() { };

// subclass sudo's ViewController
marketstar.ViewController.prototype = Object.create(sudo.ViewController.prototype);

/**
 * Override the templatesReady method
 *
 * @method templatesReady
 * @override
 */
marketstar.ViewController.prototype.templatesReady = function() {
    this.trackPageView();
    this.renderNavigation();
    this.navMenuControl();
    this.navMenuLinkHandler();
    if (typeof iScroll !== "undefined") this.bindScroller();

    if (this.onTemplatesReady) this.onTemplatesReady();
};

/**
 * Track Google Analytics PageView
 *
 * @method trackPageView
 */
marketstar.ViewController.prototype.trackPageView = function() {
    $.getScript('js/lib/GALocalStorage.js').done(function() {
        if (!ga_storage) return;
        var pageName = location.pathname.substring(location.pathname.lastIndexOf("/") + 1);
        ga_storage._setAccount(Config.googleAnalyticsTrackingId);
        ga_storage._trackPageview('mobile/'+pageName);
    });
};

/**
 * bind iScroller to controller
 *
 * @method bindScroller
 */
marketstar.ViewController.prototype.bindScroller = function() {
    if (document.getElementById('iscroller')) {
        this.iScroll = new iScroll('iscroller', {
            useTransition: false,
            checkDOMChanges: true,
            hScrollbar: false,
            vScrollbar: false,

            onBeforeScrollStart: function (e) {
                e.stopPropagation();

                var nodeType = e.explicitOriginalTarget ? e.explicitOriginalTarget.nodeName.toLowerCase():(e.target ? e.target.nodeName.toLowerCase():'');

                if(nodeType !='select' && nodeType !='option' && nodeType !='input' && nodeType!='textarea' ) {
                    e.preventDefault();
                }
            }
        });
    }   

    if(document.getElementById('navscroller')) {
        this.iScroll = new iScroll('navscroller', {
            useTransition: false,
            checkDOMChanges: true,


            onBeforeScrollStart: function (e) {
                //e.stopPropagation();
                this.maxScrollY = (parseInt($(window).height()) - parseInt($('.nav-container').height() + 20));

                var nodeType = e.explicitOriginalTarget ? e.explicitOriginalTarget.nodeName.toLowerCase() : (e.target ? e.target.nodeName.toLowerCase() : '');

                if (nodeType != 'select' && nodeType != 'option' && nodeType != 'input' && nodeType != 'textarea') {
                    e.preventDefault();
                }
            }
        });
    }

    if(document.getElementById('filterscroller')) {
        this.iScroll = new iScroll('filterscroller', {
            useTransition: false,
            checkDOMChanges: true,


            onBeforeScrollStart: function (e) {
                e.stopPropagation();
                this.maxScrollY = (parseInt($(window).height()) - parseInt($('.filter-container').height() + 20));

                var nodeType = e.explicitOriginalTarget ? e.explicitOriginalTarget.nodeName.toLowerCase() : (e.target ? e.target.nodeName.toLowerCase() : '');

                if (nodeType != 'select' && nodeType != 'option' && nodeType != 'input' && nodeType != 'textarea') {
                    e.preventDefault();
                }
            }
        });
    }
};